"""
Custom django-crispy-forms layouts.
"""
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from crispy_forms.utils import flatatt


class CradminSubmitButton(Submit):
    template = 'django_cradmin/crispylayouts/submitbutton.django.html'
    extra_button_attributes = {}

    def get_extra_button_attributes(self):
        return self.extra_button_attributes

    def __init__(self, name, value, icon_cssclass=None, **kwargs):
        self.extra_button_attributes = flatatt(self.get_extra_button_attributes())
        self.icon_cssclass = icon_cssclass
        super(CradminSubmitButton, self).__init__(name, value, **kwargs)


class PrimarySubmit(CradminSubmitButton):
    field_classes = "btn btn-primary"


class DangerSubmit(CradminSubmitButton):
    field_classes = "btn btn-danger"


class DefaultSubmit(CradminSubmitButton):
    field_classes = "btn btn-default"


class CradminFormHelper(FormHelper):
    template = 'django_cradmin/crispylayouts/cradmin_form_helper.django.html'



from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView
from django.http import HttpResponseRedirect

from django_cradmin.registry import cradmin_instance_registry


class RoleSelectView(ListView):
    paginate_by = 30
    template_name = 'django_cradmin/roleselect.django.html'
    context_object_name = 'roles'
    pagetitle = _('What would you like to edit?')
    autoredirect_if_single_role = True
    list_cssclass = 'django-cradmin-roleselect-list django-cradmin-roleselect-list-flat'

    def get_queryset(self):
        cradmin_instance = cradmin_instance_registry.get_current_instance(self.request)
        return cradmin_instance.get_rolequeryset()

    def get(self, *args, **kwargs):
        cradmin_instance = cradmin_instance_registry.get_current_instance(self.request)
        if self.get_autoredirect_if_single_role() and self.get_queryset().count() == 1:
            only_role = self.get_queryset().first()
            return HttpResponseRedirect(cradmin_instance.rolefrontpage_url(
                cradmin_instance.get_roleid(only_role)))
        else:
            # Add cradmin_instance to request just like django_cradmin.decorators.cradminview.
            # Convenient when overriding standalone-base.django.html and using the current
            # CrInstance to distinguish multiple crinstances.
            self.request.cradmin_instance = cradmin_instance
            return super(RoleSelectView, self).get(*args, **kwargs)

    def get_autoredirect_if_single_role(self):
        return self.autoredirect_if_single_role

    def get_pagetitle(self):
        return self.pagetitle

    def get_list_cssclass(self):
        return self.list_cssclass

    def get_context_data(self, **kwargs):
        context = super(RoleSelectView, self).get_context_data(**kwargs)
        context['pagetitle'] = self.get_pagetitle()
        context['list_cssclass'] = self.get_list_cssclass()
        return context



from django.contrib import messages
from django.views.generic import DeleteView as DjangoDeleteView
from django.utils.translation import ugettext_lazy as _


class DeleteView(DjangoDeleteView):

    #: The name of the template to use.
    template_name = 'django_cradmin/viewhelpers/delete.django.html'

    def get_object_preview(self):
        """
        The preview of the object. Used when asking the user if he/she wants to
        delete the current object.
        """
        obj = self.get_object()
        return unicode(obj)

    def get_success_url(self):
        """
        Get the URL to go to if object was deleted.

        Defaults to the INDEX view of the current app.
        """
        return self.request.cradmin_app.reverse_appindexurl()

    def get_context_data(self, **kwargs):
        context = super(DeleteView, self).get_context_data(**kwargs)
        obj = context['object']
        context['model_verbose_name'] = obj._meta.verbose_name
        context['success_url'] = self.get_success_url()
        context['object_preview'] = self.get_object_preview()
        return context

    def get_queryset_for_role(self, role):
        """
        Get a queryset with all objects of :obj:`.model`  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role(self.request.cradmin_role)
        return queryset

    def get_success_message(self, object_preview):
        """
        Override this to provide a success message.

        Used by :meth:`.add_success_messages`.
        """
        return _('Deleted "%(what)s"') % {
            'what': object_preview
        }

    def add_success_messages(self, object_preview):
        """
        Called after the object has been deleted.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(object_preview)
        if success_message:
            messages.success(self.request, success_message)

    def delete(self, request, *args, **kwargs):
        object_preview = self.get_object_preview()
        response = super(DeleteView, self).delete(request, *args, **kwargs)
        self.add_success_messages(object_preview)
        return response



from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required
from django_cradmin.apps.cradmin_temporaryfileuploadstore.views.temporary_file_upload_api import \
    UploadTemporaryFilesView

urlpatterns = patterns(
    '',
    url(r'^temporary_file_upload_api$',
        login_required(UploadTemporaryFilesView.as_view()),
        name='cradmin_temporary_file_upload_api'),
)



from django.template.loader import render_to_string


class MenuItem(object):
    """
    A  menu item. Basically a pythonic interface to
    create a HTML element.

    If you want to make major changes to the look and feel of the
    menu, you may want to subclass this and override the template
    and/or :meth:`render` method.
    """
    template_name = 'django_cradmin/menuitem.django.html'

    def __init__(self, label, url, icon='circle-o', active=False, attributes={}):
        """
        Parameters:
            label: A label shown in the menu.
            url: The url to go to whem the user clicks the menu item.
            icon: The name of a font-awesome icon (E.g.: "database", "user", ...).
            active: Should be ``True`` if the menuitem should be styled as active.
        """
        self.label = label
        self.url = url
        self.icon = icon
        self.attributes = attributes
        self.active = active

    def render(self):
        return render_to_string(self.template_name, {
            'menuitem': self
        })

    def is_active(self):
        return self.active


class Menu(object):
    """
    Base class for the menu.

    You subclass this for your site, and set
    :obj:`django_cradmin.crinstance.BaseInstance.menuclass`.

    In advanced cases, you may create multiple subclasses of
    this for your site, and override
    :obj:`django_cradmin.crinstance.BaseInstance.get_menuclass`

    Attributes:
        cradmin_instance (BaseInstance): The current cradmin instance.
        request (HttpRequest): Shortcut for ``cradmin_instance.request``.
        menu (list): A list of MenuItem objects. You use :meth:`.add`
            to add items to the menu.
    """

    #: The name of the template to use for rendering the menu.
    #: Used by :meth:`.get_template_name`.
    template_name = "django_cradmin/menu.django.html"

    def __init__(self, cradmin_instance):
        """
        Parameters:
            cradmin_instance (django_cradmin.crinstance.BaseInstance):
        """
        self.cradmin_instance = cradmin_instance
        self.request = cradmin_instance.request
        self.headeritems = []
        self.mainitems = []
        self.footeritems = []
        self.build_menu()

    def add(self, *args, **kwargs):
        """
        Add menu item.

        args and kwargs are forwarded to the menu class (see :meth:`.get_menuitem_class`).
        """
        menuitemclass = self.get_menuitem_class()
        self.mainitems.append(
            menuitemclass(*args, **kwargs)
        )

    def get_menuitem_class(self):
        """
        Get the class used to render each menu item.

        Returns:
            :class:`.MenuItem` by default.
        """
        return MenuItem

    def add_headeritem(self, *args, **kwargs):
        """
        Add item to the menu header.

        args and kwargs are forwarded to the menu class (see :meth:`.get_headeritem_class`).
        """
        headeritem_class = self.get_headeritem_class()
        self.headeritems.append(
            headeritem_class(*args, **kwargs)
        )

    def get_headeritem_class(self):
        """
        Get the class used to render each item in the menu header.

        Returns:
            :class:`.MenuItem` by default.
        """
        return MenuItem

    def add_footeritem(self, *args, **kwargs):
        """
        Add item to the menu footer.

        args and kwargs are forwarded to the menu class (see :meth:`.get_footeritem_class`).
        """
        footeritem_class = self.get_footeritem_class()
        self.footeritems.append(
            footeritem_class(*args, **kwargs)
        )

    def get_footeritem_class(self):
        """
        Get the class used to render each item in the menu footer.

        Returns:
            :class:`.MenuItem` by default.
        """
        return MenuItem

    def build_menu(self):
        """
        Build the menu. You HAVE to override this.

        Example::

            def build_menu(self):
                self.add(label='Home', url='/myadmin/')
                self.add(label='My menu item', url='/myadmin/my/item')
        """
        raise NotImplementedError()

    def __iter__(self):
        """
        Iterate over all items in the menu.

        Returns:
            An iterator over subclasses of :class:`.MenuItem`.
        """
        return iter(self.menu)

    def appindex_url(self, appname):
        """
        Shortcut for ``self.cradmin_instance.appindex_url(...)``.

        See :meth:`django_cradmin.BaseInstance.appindex_url`.
        """
        return self.cradmin_instance.appindex_url(appname)

    def roleselectview_url(self):
        """
        Shortcut for ``self.cradmin_instance.roleselectview_url()``.

        See :meth:`django_cradmin.BaseInstance.roleselectview_url`.
        """
        return self.cradmin_instance.roleselectview_url()

    def get_template_name(self):
        """
        Get the name of the template to use for rendering the menu.

        Defaults to :obj:`.template_name`.
        """
        return self.template_name

    def render(self, context):
        """
        Render the menu.

        Returns:
            The menu as HTML.
        """
        return render_to_string(self.get_template_name(), context)



from django.template import Library
from django_cradmin import imageutils
import logging


register = Library()
logger = logging.getLogger(__name__)


@register.simple_tag
def cradmin_thumbnail(*args, **kwargs):
    """
    Template tag that forwards all arguments to
    :meth:`django_cradmin.imageutils.request_thumbnail`.
    """
    try:
        return imageutils.request_thumbnail(*args, **kwargs).url
    except:
        logger.exception('cradmin_thumbnail failed. args=%r, kwargs=%r', args, kwargs)
        return ''


@register.simple_tag
def cradmin_archiveimage_thumbnail(archiveimage, **kwargs):
    """
    Template tag that creates a thumbnail for the given
    :class:`django_cradmin.apps.cradmin_imagearchive.models.ArchiveImage`.

    Parameters:
        archiveimage: An ArchiveImage object.
        kwargs: Same as for :meth:`django_cradmin.imageutils.request_thumbnail`.
    """
    image = archiveimage.image
    try:
        return imageutils.request_thumbnail(image, **kwargs).url
    except:
        logger.exception('cradmin_thumbnail failed. archiveimage=%r, kwargs=%r', archiveimage, kwargs)
        return ''



"""
An example app using Sortable.
"""
from django.db import models
from django_cradmin.sortable.models import SortableBase
from django_cradmin.sortable.models import SortableManagerBase


class ItemContainer(models.Model):
    """
    A test class for being a container for items that should be sorted.

    It has a name field only for testing purposes.
    """
    name = models.CharField(
        max_length=255,
        blank=True,
        null=False,
        default='')

    def __unicode__(self):
        return u'Item container {}, {}'.format(self.id, self.name)


class SortableItemManager(SortableManagerBase):
    """
    Sortable items that inherit SortableBase must also have a manager that inherits SortableManagerBase.

    The `parent_attribute` must be set, and it must have the name of the parent in which the items belong.
    """
    parent_attribute = 'container'


class SortableItem(SortableBase):
    """
    The sortable item. The important thing here is the usage og SortableItemManager.

    The `sort_index` field is inherited from SortableBase.
    """
    container = models.ForeignKey(ItemContainer, blank=False, null=False)
    name = models.CharField(
        max_length=255,
        blank=True,
        null=False,
        default='')
    objects = SortableItemManager()

    def __unicode__(self):
        return u'Id: {}, Sort index: {}, Name: {}, {}'.format(
            self.id,
            self.sort_index,
            self.name,
            self.container
        )



from django.test import TestCase
from django.core.urlresolvers import reverse

from django_cradmin.tests.views.helpers import create_testuser


class TestLogin(TestCase):
    def setUp(self):
        self.testuser = create_testuser(username='testuser')
        self.url = reverse('django_cradmin-login')

    def test_login_ok(self):
        response = self.client.post(self.url, {
            'username': 'testuser',
            'password': 'test'
        })

        self.assertEquals(response.status_code, 302)

    def test_login_invalid(self):
        response = self.client.post(self.url, {
            'username': 'testuser',
            'password': 'invalid'
        })
        self.assertEquals(response.status_code, 200)
        self.assertIn("Your email and password didn&#39;t match", response.content)



from django.core.files.base import ContentFile
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase, RequestFactory
import htmls
import mock

from django_cradmin.apps.cradmin_imagearchive import cradminviews
from django_cradmin.apps.cradmin_imagearchive.models import ArchiveImage
from django_cradmin.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile
from django_cradmin.django_cradmin_testapp.models import TstRole
from django_cradmin.tests.helpers import create_user
from .helpers import create_image


class TestCreateView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_get_render(self):
        request = self.factory.get('/test')
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_instance.__getitem__.return_value = None  # This avoids rendering the menu
        request.cradmin_role = None

        response = cradminviews.ArchiveImageCreateView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('input[type=file][name=image]'))
        self.assertTrue(selector.exists('textarea#id_description'))

    def test_post_ok(self):
        testimage = create_image(200, 100)
        request = self.factory.post('/test', {
            'image': SimpleUploadedFile('testname.png', testimage),
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = TstRole.objects.create()
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 0)
        response = cradminviews.ArchiveImageCreateView.as_view()(request)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')
        self.assertEqual(ArchiveImage.objects.count(), 1)
        created_image = ArchiveImage.objects.first()
        self.assertEqual(created_image.image.read(), testimage)
        self.assertEqual(created_image.name, 'testname')
        self.assertEqual(created_image.description, '')

    def test_post_allfields(self):
        testimage = create_image(200, 100)
        request = self.factory.post('/test', {
            'image': SimpleUploadedFile('testname.png', testimage),
            'name': 'My name',
            'description': 'My description',
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = TstRole.objects.create()
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 0)
        response = cradminviews.ArchiveImageCreateView.as_view()(request)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')
        self.assertEqual(ArchiveImage.objects.count(), 1)
        created_image = ArchiveImage.objects.first()
        self.assertEqual(created_image.image.read(), testimage)
        self.assertEqual(created_image.description, 'My description')

    def test_post_nofile(self):
        request = self.factory.post('/test', {})
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_role = TstRole.objects.create()
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 0)
        response = cradminviews.ArchiveImageCreateView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        self.assertEqual(ArchiveImage.objects.count(), 0)
        response.render()
        selector = htmls.S(response.content)
        self.assertIn(
            'This field is required',
            selector.one('#django_cradmin_contentwrapper #div_id_image').alltext_normalized)


class TestUpdateView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.role = TstRole.objects.create()
        self.archiveimage = ArchiveImage.objects.create(role=self.role)
        self.testimage = create_image(200, 100)
        self.archiveimage.image.save('testimage.png', ContentFile(self.testimage))

    def test_get_render(self):
        request = self.factory.get('/test')
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_role = self.role

        response = cradminviews.ArchiveImageUpdateView.as_view()(request, pk=self.archiveimage.pk)
        self.assertEquals(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('input[type=text][name=name]'))
        self.assertTrue(selector.exists('input[type=file][name=image]'))
        self.assertTrue(selector.exists('textarea#id_description'))

    def test_post_ok(self):
        request = self.factory.post('/test', {
            'name': 'Updated name'
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = self.role
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 1)
        response = cradminviews.ArchiveImageUpdateView.as_view()(request, pk=self.archiveimage.pk)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')
        self.assertEqual(ArchiveImage.objects.count(), 1)
        update_image = ArchiveImage.objects.first()
        self.assertEqual(update_image.image.read(), self.testimage)
        self.assertEqual(update_image.name, 'Updated name')
        self.assertEqual(update_image.description, '')

    def test_post_allfields(self):
        testimage = create_image(200, 100)
        request = self.factory.post('/test', {
            'image': SimpleUploadedFile('testname.png', testimage),
            'name': 'My name',
            'description': 'My description',
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = self.role
        request._messages = mock.MagicMock()

        response = cradminviews.ArchiveImageUpdateView.as_view()(request, pk=self.archiveimage.pk)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')
        update_image = ArchiveImage.objects.first()
        self.assertEqual(update_image.image.read(), testimage)
        self.assertEqual(update_image.name, 'My name')
        self.assertEqual(update_image.description, 'My description')

    def test_post_noname(self):
        testimage = create_image(200, 100)
        request = self.factory.post('/test', {
            'image': SimpleUploadedFile('testname.png', testimage),
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_role = self.role
        request._messages = mock.MagicMock()

        response = cradminviews.ArchiveImageUpdateView.as_view()(request, pk=self.archiveimage.pk)
        self.assertEquals(response.status_code, 302)
        update_image = ArchiveImage.objects.first()
        self.assertEqual(update_image.name, 'testname')


class TestArchiveImageBulkAddView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testuser = create_user('testuser')

    def test_get_render(self):
        request = self.factory.get('/test')
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_instance.__getitem__.return_value = None  # This avoids rendering the menu
        request.cradmin_role = None

        response = cradminviews.ArchiveImageBulkAddView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('#django_cradmin_imagearchive_bulkadd_form'))
        self.assertTrue(selector.exists('input[type=hidden][name=filecollectionid]'))
        self.assertTrue(selector.exists('#div_id_filecollectionid'))

    def test_post_no_filecollectionid(self):
        request = self.factory.post('/test', {})
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_role = TstRole.objects.create()

        self.assertEqual(ArchiveImage.objects.count(), 0)
        response = cradminviews.ArchiveImageBulkAddView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('#div_id_filecollectionid.has-error'))
        self.assertIn(
            'You must upload at least one file.',
            selector.one('#div_id_filecollectionid').alltext_normalized)

    def test_post_single_image(self):
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(
            collection=collection,
            filename='testfile.png')
        temporaryfile.file.save('testfile.png', ContentFile(testimage))

        request = self.factory.post('/test', {
            'filecollectionid': collection.id
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.user = self.testuser
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = TstRole.objects.create()
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 0)
        response = cradminviews.ArchiveImageBulkAddView.as_view()(request)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')
        self.assertEqual(ArchiveImage.objects.count(), 1)
        created_image = ArchiveImage.objects.first()
        self.assertEqual(created_image.image.read(), testimage)
        self.assertEqual(created_image.name, 'testfile.png')
        self.assertEqual(created_image.description, '')

    def test_post_deletes_collection(self):
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(
            collection=collection,
            filename='testfile.png')
        temporaryfile.file.save('testfile.png', ContentFile(testimage))

        request = self.factory.post('/test', {
            'filecollectionid': collection.id
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.user = self.testuser
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = TstRole.objects.create()
        request._messages = mock.MagicMock()

        response = cradminviews.ArchiveImageBulkAddView.as_view()(request)
        self.assertEquals(response.status_code, 302)
        self.assertFalse(TemporaryFileCollection.objects.filter(id=collection.id).exists())

    def test_post_multiple_images(self):
        testimage1 = create_image(200, 100)
        testimage2 = create_image(100, 100)
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(
            collection=collection,
            filename='testfile1.png')
        temporaryfile.file.save('testfile1.png', ContentFile(testimage1))
        temporaryfile = TemporaryFile(
            collection=collection,
            filename='testfile2.png')
        temporaryfile.file.save('testfile2.png', ContentFile(testimage2))

        request = self.factory.post('/test', {
            'filecollectionid': collection.id
        })
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_app.reverse_appurl.return_value = '/success'
        request.cradmin_role = TstRole.objects.create()
        request.user = self.testuser
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 0)
        response = cradminviews.ArchiveImageBulkAddView.as_view()(request)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')
        self.assertEqual(ArchiveImage.objects.count(), 2)
        created_images = ArchiveImage.objects.order_by('name')

        self.assertEqual(created_images[0].image.read(), testimage1)
        self.assertEqual(created_images[0].name, 'testfile1.png')
        self.assertEqual(created_images[0].description, '')

        self.assertEqual(created_images[1].image.read(), testimage2)
        self.assertEqual(created_images[1].name, 'testfile2.png')
        self.assertEqual(created_images[1].description, '')



from django import forms


class WysiHtmlTextArea(forms.widgets.Textarea):
    template = ''  # TODO: make a template..

    def render(self, name, value, attrs=None):
        baseVal = super(WysiHtmlTextArea, self).render(name, value, attrs)

        # return render_to_string(template, {'textarea': baseVal}) #TODO: implement this instead on the above
        return u'<div django_cradmin_wysihtml>{}</div>'.format(baseVal)



import mock
import htmls
from django.test import TestCase
from django.test.client import RequestFactory
from django import http
from django import forms

from django_cradmin.viewhelpers import multiselect


class TestMultiSelectView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_object_selection_valid(self):
        class SimpleMultiSelectView(multiselect.MultiSelectView):
            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                item1 = mock.MagicMock()
                item1.pk = 1
                item2 = mock.MagicMock()
                item2.pk = 10
                item3 = mock.MagicMock()
                item3.pk = 12
                queryset.filter.return_value = [item1, item2, item3]
                return queryset

            def object_selection_valid(self):
                return http.HttpResponse('OK')

            def object_selection_invalid(self, form):
                return http.HttpResponse(form.errors.as_text())

        request = self.factory.post('/test', {
            'selected_objects': [1, 10]
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.content, "OK")

    def test_object_selection_invalid(self):
        class SimpleMultiSelectView(multiselect.MultiSelectView):
            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                return queryset

        request = self.factory.post('/test', {
            'selected_objects': [1, 10]
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        selector = htmls.S(response.content)
        self.assertEquals(
            selector.one('.alert.alert-danger').text_normalized,
            'Invalid selection. This is usually caused by someone else changing permissions '
            'while you where selecting items to edit.')

    def test_object_selection_invalid_override(self):
        class SimpleMultiSelectView(multiselect.MultiSelectView):
            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                return queryset

            def object_selection_invalid(self, form):
                return http.HttpResponse(form.errors.as_text())

        request = self.factory.post('/test', {
            'selected_objects': [1, 10]
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        self.assertIn('1 is not one of the available choices.', response.content)


class TestMultiSelectFormView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_first_load(self):
        class DemoForm(forms.Form):
            data = forms.CharField()

        class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
            form_class = DemoForm
            model = mock.MagicMock()

            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                item1 = mock.MagicMock()
                item1.pk = 1
                queryset.filter.return_value = [item1]
                return queryset

            def get_field_layout(self):
                return ['data']

        request = self.factory.post('/test', {
            'selected_objects': [1],
            'is_the_multiselect_form': 'yes'
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEquals(selector.count('#django_cradmin_contentwrapper form'), 1)
        self.assertEquals(selector.one('input[name=selected_objects]')['type'], 'hidden')
        self.assertEquals(selector.one('input[name=selected_objects]')['value'], '1')
        self.assertEquals(selector.count('input[name=data]'), 1)
        self.assertFalse(selector.exists('form .has-error'))

    def test_form_invalid(self):
        class DemoForm(forms.Form):
            data = forms.CharField()

        class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
            form_class = DemoForm
            model = mock.MagicMock()

            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                item1 = mock.MagicMock()
                item1.pk = 1
                queryset.filter.return_value = [item1]
                return queryset

            def get_field_layout(self):
                return ['data']

        request = self.factory.post('/test', {
            'selected_objects': [1],
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEquals(selector.count('#django_cradmin_contentwrapper form'), 1)
        self.assertEquals(selector.one('input[name=selected_objects]')['type'], 'hidden')
        self.assertEquals(selector.one('input[name=selected_objects]')['value'], '1')
        self.assertEquals(selector.count('input[name=data]'), 1)
        self.assertEquals(
            selector.one('#div_id_data .help-block').alltext_normalized,
            'This field is required.')

    def test_form_valid(self):
        class DemoForm(forms.Form):
            data = forms.CharField()

        class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
            form_class = DemoForm
            model = mock.MagicMock()

            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                item1 = mock.MagicMock()
                item1.pk = 1
                queryset.filter.return_value = [item1]
                return queryset

            def form_valid(self, form):
                return http.HttpResponse('Submitted: {data}'.format(**form.cleaned_data))

        request = self.factory.post('/test', {
            'selected_objects': [1],
            'data': 'Hello world'
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        self.assertEquals(response.content, 'Submitted: Hello world')

    def test_form_valid_success_redirect(self):
        class DemoForm(forms.Form):
            data = forms.CharField()

        class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
            form_class = DemoForm
            model = mock.MagicMock()

            def get_queryset_for_role(self, role):
                queryset = mock.MagicMock()
                item1 = mock.MagicMock()
                item1.pk = 1
                queryset.filter.return_value = [item1]
                return queryset

            def get_success_url(self):
                return '/success'

            def form_valid(self, form):
                return self.success_redirect_response()

        request = self.factory.post('/test', {
            'selected_objects': [1],
            'data': 'Hello world'
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['Location'], '/success')



import os
import posixpath
from django.conf import settings
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _


def archiveimage_upload_to(archiveimage, filename):
    if archiveimage.id is None:
        raise ValueError('Can not set image until after the ArchiveImage object has been created.')
    name, extension = os.path.splitext(filename)
    filenamepattern = getattr(settings, 'DJANGO_CRADMIN_IMAGEARCHIVE_FILENAMEPATTERN',
                              'cradmin_imagearchive_images/{id}{extension}')
    return filenamepattern.format(
        id=archiveimage.id,
        extension=extension)


class ArchiveImageManager(models.Manager):
    def filter_owned_by_role(self, role):
        return self.get_queryset().filter(
            role_object_id=role.id,
            role_content_type=ContentType.objects.get_for_model(role.__class__))


class ArchiveImage(models.Model):
    objects = ArchiveImageManager()

    role_content_type = models.ForeignKey(
        ContentType,
        verbose_name=_('role'),
        help_text=_('The role owning this image.'))
    role_object_id = models.PositiveIntegerField()
    role = GenericForeignKey('role_content_type', 'role_object_id')

    #: The image.
    image = models.ImageField(
        max_length=255, null=True, blank=False,
        height_field='image_height',
        width_field='image_width',
        verbose_name=_('image'),
        help_text=_('Select an image to add to the archive.'),
        upload_to=archiveimage_upload_to)

    #: The height of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_height = models.IntegerField(null=True, blank=True, editable=False)

    #: The width of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_width = models.IntegerField(null=True, blank=True, editable=False)

    #: The file extension
    file_extension = models.CharField(
        max_length=255, blank=False, null=False, editable=False)

    #: The name of the image file.
    name = models.CharField(
        max_length=255, blank=True, null=False,
        verbose_name=_('name'),
        help_text=_(
            'Give the image a name (optional). If you leave this blank, the name of the uploaded image file is used.'
        )
    )

    #: An optional description of the image.
    description = models.TextField(
        blank=True, null=False, default='',
        verbose_name=_('description'),
        help_text=_(
            'An optional description of the image. Think if this as a description '
            'of the image for visually impaired users. This means that you should describe '
            'the information carried in the image (if any).'),
    )

    #: Create datetime.
    created_datetime = models.DateTimeField(
        editable=False,
        auto_now_add=True
    )

    class Meta:
        verbose_name = _('archive image')
        verbose_name_plural = _('archive images')

    def clean(self):
        if not self.name:
            if self.image:
                self.name = os.path.splitext(posixpath.basename(self.image.name))[0]
        if self.name:
            self.file_extension = os.path.splitext(self.name)[1]

    def __unicode__(self):
        return self.name

    def __repr__(self):
        return 'ArchiveImage(name={name}, image_width={image_width}, image_height={image_height})'.format(
            name=self.name.encode('ascii', 'replace'),
            image_width=self.image_width,
            image_height=self.image_height)

    def get_preview_html(self):
        return render_to_string('django_cradmin/apps/cradmin_imagearchive/preview.django.html', {
            'archiveimage': self
        })



import urllib
from django import forms
from django import http
from django.contrib import messages
from django.contrib.contenttypes.generic import GenericForeignKey
from django.core import serializers
from django.utils.translation import ugettext_lazy as _

from crispy_forms import layout

from . import formbase


class CreateUpdateViewMixin(formbase.FormViewMixin):
    """
    Mixin class for Update and Create views.
    """

    #: The model class.
    model = None

    #: List of field names.
    #: See :meth:`.get_field_layout`.
    fields = None

    #: The name of the submit button used for preview.
    #: Only used when :meth:`.preview_url` is defined.
    submit_preview_name = 'submit-preview'

    #: The field that should always be set to the current role.
    #: Removes the field from the form (see :meth:`.get_form`),
    #: and instead sets the value directly on the object in
    #: :meth:`.save_object`.
    roleid_field = None

    #: The viewname within this app for the edit view.
    #: See :meth:`.get_editurl`.
    editview_appurl_name = 'edit'

    def get_field_layout(self):
        """
        Get a list/tuple of fields. These are added to a ``crispy_forms.layout.Layout``.
        Defaults to a all fields on the model. If :obj:`.fields`, we use those fields.

        Simple example (same as specifying the fields in :obj:`.fields`)::

            from crispy_forms import layout

            class MyCreateView(CreateView):
                def get_field_layout(self):
                    return [
                        layout.Div(
                            'title', 'name', 'size', 'tags',
                            css_class='cradmin-globalfields')
                    ]

        A slightly more complex example::

            from crispy_forms import layout

            class MyCreateView(CreateView):
                def get_field_layout(self):
                    return [
                        layout.Div('title', css_class="cradmin-focusfield cradmin-focusfield-lg"),
                        layout.Div('description', css_class="cradmin-focusfield"),
                        layout.Fieldset('Metadata',
                            'size',
                            'tags'
                        )

                    ]

        """
        fields = forms.fields_for_model(self.model, fields=self.fields).keys()
        if self.roleid_field and self.roleid_field in fields:
            fields.remove(self.roleid_field)
        return [layout.Div(*fields, css_class='cradmin-globalfields')]

    def get_form(self, form_class):
        """
        If you set :obj:`.roleid_field`, we will remove that field from
        the form.

        .. note::
            The :obj:`.roleid_field` handling also works for GenericForeignKey
            fields (removes the content type and object pk field from the form).
        """
        form = super(CreateUpdateViewMixin, self).get_form(form_class)
        if self.roleid_field:
            roleid_fieldobject = getattr(self.model, self.roleid_field, None)
            if isinstance(roleid_fieldobject, GenericForeignKey):
                for fieldname in roleid_fieldobject.fk_field, roleid_fieldobject.ct_field:
                    if fieldname in form.fields:
                        del form.fields[fieldname]
            else:
                if self.roleid_field in form.fields:
                    del form.fields[self.roleid_field]
        return form

    def get_context_data(self, **kwargs):
        context = super(CreateUpdateViewMixin, self).get_context_data(**kwargs)
        context['model_verbose_name'] = self.model._meta.verbose_name
        if getattr(self, 'show_preview', False):
            context['preview_url'] = self.get_preview_url()
            context['show_preview'] = True
        return context

    def get_editurl(self, obj):
        """
        Get the edit URL for ``obj``.

        Defaults to::

            self.request.cradmin_app.reverse_appurl(self.editview_appurl_name, args=[obj.pk])
        """
        return self.request.cradmin_app.reverse_appurl(self.editview_appurl_name, args=[obj.pk])

    def _get_full_editurl(self, obj):
        url = self.get_editurl(obj)
        if 'success_url' in self.request.GET:
            url = '{}?{}'.format(
                url, urllib.urlencode({
                    'success_url': self.request.GET['success_url']}))
        return url

    def get_success_url(self):
        if 'submit-save-and-continue-editing' in self.request.POST:
            return self._get_full_editurl(self.object)
        else:
            return self.get_default_save_success_url()

    def save_object(self, form, commit=True):
        """
        Save the object. You can override this to customize how the
        form is turned into a saved object.

        Make sure you call ``super`` if you override this (see the docs for the commit parameter).
        If you do not, you will loose the automatic handling of obj:`.roleid_field`.

        Parameters:
            commit (boolean): If this is ``False``, the object is returned
                unsaved. Very useful when you want to manipulate the object
                before saving it in a subclass.

        Returns:
            The saved object.
        """
        obj = form.save(commit=False)
        if self.roleid_field:
            setattr(obj, self.roleid_field, self.request.cradmin_role)
        if commit:
            obj.save()
        return obj

    def form_valid(self, form):
        """
        If the form is valid, save the associated model.
        """
        if self.preview_requested():
            self._store_preview_in_session(self.serialize_preview(form))
            self.show_preview = True
            return self.render_to_response(self.get_context_data(form=form))
        else:
            self.object = self.save_object(form)
            self.form_saved(self.object)
            self.add_success_messages(self.object)
            return http.HttpResponseRedirect(self.get_success_url())

    def form_saved(self, object):
        """
        Called after the form has been successfully saved.
        The ``object`` is the saved object.

        Does nothing by default, but you can override it if you need to
        do something extra post save.
        """
        pass

    def get_success_message(self, object):
        """
        Override this to provide a success message.

        The ``object`` is the saved object.

        Used by :meth:`.add_success_messages`.
        """
        return None

    def add_success_messages(self, object):
        """
        Called after the form has been saved, and after :meth:`.form_saved` has been called.

        The ``object`` is the saved object.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(object)
        if success_message:
            messages.success(self.request, success_message)

    def get_form_invalid_message(self, form):
        """
        You can override this to provide a custom error message.

        Defaults to "Please fix the errors in the form below.".

        The ``form`` is the invalid form object.

        Used by :meth:`.add_form_invalid_messages`.
        """
        return _('Please fix the errors in the form below.')

    def add_form_invalid_messages(self, form):
        """
        Called to add messages when the form does not validate.

        The ``form`` is the invalid form object.

        Defaults to add :meth:`.get_form_invalid_message` as a django messages
        error message if :meth:`.get_form_invalid_message` returns anything.

        You can override this to add multiple messages or to show error messages in some other way.
        """
        form_invalid_message = self.get_form_invalid_message(object)
        if form_invalid_message:
            messages.error(self.request, form_invalid_message)

    def form_invalid(self, form):
        self.add_form_invalid_messages(form)
        return super(CreateUpdateViewMixin, self).form_invalid(form)

    def preview_requested(self):
        """
        Determine if a preview was requested.

        Defaults to checking if :obj:`.submit_preview_name` is
        in ``request.POST``.
        """
        return self.submit_preview_name in self.request.POST

    def get_preview_url(self):
        """
        Get the URL of the preview view.
        """
        return None

    def _store_preview_in_session(self, data):
        self.request.session[self.__class__.get_preview_sessionkey()] = data

    def serialize_preview(self, form):
        """
        Seralize for preview.

        Defaults to serializing the object as JSON using ``django.core.serializers``.
        You can safely override this, but you will also have to override
        :meth:`deserialize_preview`.
        """
        return serializers.serialize('json', [self.save_object(form, commit=False)])

    @classmethod
    def deserialize_preview(self, serialized):
        """
        Deseralize a preview serialized with :meth:`.serialize_preview`.
        """
        return list(serializers.deserialize('json', serialized))[0].object

    @classmethod
    def get_preview_sessionkey(cls):
        """
        Get the session key used for preview. You should not
        need to override this.
        """
        return 'django_cradmin__{module}.{classname}'.format(
            module=cls.model.__module__,
            classname=cls.model.__name__)

    @classmethod
    def get_preview_data(cls, request):
        """
        Get the preview data.

        You should use this in the preview view to get the
        data for the preview.

        You normally do not override this. If you want to manage
        serialization yourself, see :meth:`.serialize_preview`.
        """
        serialized = request.session.pop(cls.get_preview_sessionkey())
        return cls.deserialize_preview(serialized)



from crispy_forms import layout
from django.views.generic import FormView as DjangoFormView

from django_cradmin import crapp
from django_cradmin.crispylayouts import CradminFormHelper


class FormViewMixin(object):
    """
    Mixin class for Update and Create views.
    """

    #: Get the view name for the listing page.
    #: You can set this, or implement :meth:`.get_listing_url`.
    #: Defaults to :obj:`django_cradmin.crapp.INDEXVIEW_NAME`.
    listing_viewname = crapp.INDEXVIEW_NAME

    #: Used to add custom attributes like angularjs directives to the form.
    #: See :meth:`.get_form_attributes`.
    form_attributes = {}

    #: A list of extra form css classes. See :meth:`.get_extra_form_css_classes`.
    extra_form_css_classes = []

    #: The ID to set on the DOM element for the form. See :meth:`.get_form_id`.
    form_id = None

    def get_field_layout(self):
        """
        Get a list/tuple of fields. These are added to a ``crispy_forms.layout.Layout``.

        Must be overridden.

        Simple example::

            from django.views.generic import FormView
            from crispy_forms import layout

            class MyFormView(FormViewMixin, FormView):
                def get_field_layout(self):
                    return [
                        layout.Div(
                            'title', 'name', 'size', 'tags',
                            css_class='cradmin-globalfields')
                    ]

        A slightly more complex example::

            from crispy_forms import layout

            class MyFormView(FormViewMixin, FormView):
                def get_field_layout(self):
                    return [
                        layout.Div('title', css_class="cradmin-focusfield cradmin-focusfield-lg"),
                        layout.Div('description', css_class="cradmin-focusfield"),
                        layout.Fieldset('Metadata',
                            'size',
                            'tags'
                        )

                    ]

        """

    def get_hidden_fields(self):
        """
        Get hidden fields for the form.

        Returns:
            An iterable of :class:`crispy_forms.layout.Hidden` objects.
            Defaults to an empty list.
        """
        return []

    def get_buttons(self):
        """
        Get buttons for the form, normally one or more submit button.

        Each button must be a crispy form layout object, typically some
        subclass of :class:`crispy_forms.layout.Submit`.

        See:
            This method is used by :meth:`.get_button_layout`.
        """
        return []

    def get_button_layout(self):
        """
        Get the button layout. This is added to the crispy form layout.

        Defaults to a :class:`crispy_forms.layout.Div` with css class
        ``django_cradmin_submitrow`` containing all the buttons
        returned by :meth:`.get_buttons`.
        """
        return [
            layout.Div(*self.get_buttons(), css_class="django_cradmin_submitrow")
        ]

    def get_form_attributes(self):
        """
        You can add custom attributes to the form via this method.

        This is set as the ``attrs``-attribute of the crispy FormHelper
        created in :meth:`.get_formhelper`.

        Defaults to :obj:`.form_attributes`.

        Returns:
            A dictionary to set any kind of form attributes. Underscores in
            keys are translated into hyphens.
        """
        return self.form_attributes

    def get_form_css_classes(self):
        """
        Returns list of css classes set on the form. You normally
        want to override :meth:`.get_extra_form_css_classes` instead
        of this method unless you want to provide completely custom
        form styles.
        """
        form_css_classes = [
            'django-cradmin-form'
        ]
        form_css_classes.extend(self.get_extra_form_css_classes())
        return form_css_classes

    def get_extra_form_css_classes(self):
        """
        Returns list of extra form css classes. These classes are added to
        the default list of css classes in :meth:`.get_form_css_classes`.

        Defaults to :obj:`.extra_form_css_classes`.
        """
        return self.extra_form_css_classes

    def get_form_id(self):
        """
        Returns the ID to set on the DOM element for the form.

        Defaults to :obj:`.form_id`.
        """
        return self.form_id

    def get_formhelper(self):
        """
        Get a :class:`crispy_forms.helper.FormHelper`.

        You normally do not need to override this directly. Instead
        you should override:

        - :meth:`.get_field_layout`.
        - :meth:`.get_hidden_fields`
        - :meth:`.get_buttons` (or perhaps :meth:`.get_button_layout`)
        """
        helper = CradminFormHelper()
        helper.form_class = ' '.join(self.get_form_css_classes())
        layoutargs = list(self.get_field_layout()) + list(self.get_button_layout()) + list(self.get_hidden_fields())
        helper.layout = layout.Layout(*layoutargs)
        helper.form_action = self.request.get_full_path()
        form_id = self.get_form_id()
        if form_id:
            helper.form_id = form_id
        helper.attrs = self.get_form_attributes()
        return helper

    def get_context_data(self, **kwargs):
        context = super(FormViewMixin, self).get_context_data(**kwargs)
        context['formhelper'] = self.get_formhelper()
        return context

    def get_listing_url(self):
        """
        Get the URL of the listing view.

        Defaults to :obj:`.listing_viewname`. This is used as the success URL
        by default.
        """
        return self.request.cradmin_app.reverse_appurl(self.listing_viewname)

    def get_default_save_success_url(self):
        if 'success_url' in self.request.GET:
            return self.request.GET['success_url']
        else:
            return self.get_listing_url()

    def get_success_url(self):
        return self.get_default_save_success_url()


class FormView(FormViewMixin, DjangoFormView):
    """
    Form view with the :class:`.FormViewMixin`.

    Usage::


    """



import posixpath
from django import forms
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


class ImageWidget(forms.ClearableFileInput):
    """
    Clearable image widget.

    Can be made un-clearable (for required fields) using the
    ``clearable`` argument for ``__init__``.
    """
    template_name = 'django_cradmin/filewidgets/imagewidget.django.html'

    #: Comma separated string of filetypes that we should accept.
    #: Added to the file upload field as the accept attribute.
    #: Defaults to ``"image/png,image/jpeg,image/gif"``.
    accept = 'image/*'

    def __init__(self, attrs=None, template_name=None, clearable=True,
                 preview_width=300, preview_height=200, preview_format='auto'):
        self.clearable = clearable
        if template_name:
            self.template_name = template_name
        self.preview_width = preview_width
        self.preview_height = preview_height
        self.preview_format = preview_format
        super(ImageWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        attrs['django-cradmin-image-preview-filefield'] = ''
        if self.accept:
            attrs['accept'] = self.accept
        input_html = forms.FileInput.render(self, name, value, attrs)
        output = render_to_string(self.template_name, {
            'input_html': input_html,
            'image_path': getattr(value, 'name', None),
            'MEDIA_URL': settings.MEDIA_URL,
            'clear_checkbox_name': self.clear_checkbox_name(name),
            'clearable': self.clearable,
            'preview_width': self.preview_width,
            'preview_height': self.preview_height,
            'preview_format': self.preview_format
        })
        return mark_safe(output)


class FileWidget(forms.ClearableFileInput):
    """
    Clearable file widget.

    Can be made un-clearable (for required fields) using the
    ``clearable`` argument for ``__init__``.
    """
    template_name = 'django_cradmin/filewidgets/filewidget.django.html'

    #: Comma separated string of filetypes that we should accept.
    #: Added to the file upload field as the accept attribute.
    #: Defaults to ``None``, which means that all filetypes are allowed.
    #: Example: ``"application/pdf,application/msword,text/plain"``.
    accept = None

    def __init__(self, attrs=None, template_name=None, clearable=True):
        self.clearable = clearable
        if template_name:
            self.template_name = template_name
        super(FileWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        if self.accept:
            attrs['accept'] = self.accept
        input_html = forms.FileInput.render(self, name, value, attrs)
        file_path = getattr(value, 'name', None)
        if file_path:
            file_name = posixpath.basename(file_path)
        else:
            file_name = None
        output = render_to_string(self.template_name, {
            'input_html': input_html,
            'file_path': file_path,
            'file_name': file_name,
            'MEDIA_URL': settings.MEDIA_URL,
            'clear_checkbox_name': self.clear_checkbox_name(name),
            'clearable': self.clearable,
        })
        return mark_safe(output)



from django.contrib.auth.decorators import login_required
from django.conf.urls import patterns, url, include
from django.shortcuts import render
from django.core.urlresolvers import reverse
from django.utils.html import format_html

from .registry import cradmin_instance_registry
from . import crapp
from .views import roleselect


def reverse_cradmin_url(instanceid, appname, roleid,
                        viewname=crapp.INDEXVIEW_NAME,
                        args=None, kwargs=None):
    """
    Reverse an URL within a cradmin instance.

    Usage is very similar to :func:`django.core.urlresolvers.reverse`,
    but you specify the cradmin instance, appname, roleid and viewname
    instead of the url-name

    Examples::

        myapp_index_url = reverse_cradmin_url(
            instanceid='siteadmin',
            appname='myapp',
            roleid=site.id)

        myapp_add_url = reverse_cradmin_url(
            instanceid='siteadmin',
            appname='myapp',
            roleid=site.id,
            viewname='add')
    """
    if args:
        args = [roleid] + list(args)
    else:
        if not kwargs:
            kwargs = {}
        kwargs['roleid'] = roleid

    urlname = u'{}-{}-{}'.format(instanceid, appname, viewname)
    return reverse(urlname, args=args, kwargs=kwargs)


class BaseCrAdminInstance(object):
    """
    Base class for a django_cradmin instance.

    You define a subclass of this to setup a django_cradmin instance.

    Attributes:
        request (HttpRequest): The current HttpRequest.
    """
    #: The ID of the cradmin instance. Must be unique for the Django
    #: instance/site. Must be a string.
    #: This is typically a short readable slug that describes what
    #: the cradmin instance does.
    #: You do not need to specify this except you need to communicate/link
    #: between cradmin instances.
    id = None

    #: The regex for matching the role id.
    #: Defaults to ``\d+``.
    roleid_regex = r'\d+'

    #: The menu class for this cradmin instance.
    menuclass = None

    #: The class defining the role for this cradmin instance.
    roleclass = None

    #: The name of the app that the user should be redirected to
    #: after selecting a role. Subclasses MUST eighter specify
    #: this or override :meth:`rolefrontpage_url`.
    rolefrontpage_appname = None

    #: Apps within the instance.
    #: Iterable of ``(appname, appclass)`` tuples where ``appname``
    #: is a slug for the app and ``appclass`` is a subclass of
    #: :class:`django_cradmin.crapp.App`.
    #: Can also be specified by overriding :meth:`.get_apps`.
    apps = []

    def __init__(self, request):
        """
        Parameters:
            request (HttpRequest): The current HttpRequest.
                Stored in :attr:`.request`.
        """
        self.request = request

    def get_rolequeryset(self):
        """
        Get the roles for the authenticated user.

        You get the authenticated user from ``self.request.user``.
        """
        raise NotImplementedError()

    def get_titletext_for_role(self, role):
        """
        Get a short title briefly describing the given ``role``.
        """
        raise NotImplementedError()

    def get_descriptiontext_for_role(self, role):
        """
        Get a longer description for the given ``role``.

        This is never used directly on its own - it is just the default text
        used by :meth:`.get_descriptionhtml_for_role`. If you want
        HTML in your description, override :meth:`.get_descriptionhtml_for_role`
        instead.
        """
        return None

    def get_descriptionhtml_for_role(self, role):
        """
        Get a longer description for the given ``role``.
        This is always shown after/below :meth:`.get_titletext_for_role`.

        Defaults to :meth:`.get_descriptiontext_for_role` filtered to
        make it HTML safe and wrapped in a paragraph tag.
        """
        descriptiontext = self.get_descriptiontext_for_role(role)
        if descriptiontext:
            return format_html(u'<p>{}</p>', descriptiontext)
        else:
            return ''

    def get_roleid(self, role):
        """
        Get the ID for the given ``role``.
        """
        return role.pk

    def get_role_from_roleid(self, roleid):
        """
        Get the role for the given ``roleid``.

        Defaults to looking up a :obj:`roleclass` object where
        ``pk==roleid``.

        Returns:
            A role object or ``None``.
        """
        try:
            return self.roleclass.objects.get(pk=roleid)
        except self.roleclass.DoesNotExist:
            pass
        return None

    def invalid_roleid_response(self, roleid):
        """
        This is called whenever someone requests a role slug that does not exist
        (if :meth:.`get_role_from_roleid`) returns ``None``.

        Returns:
            django.http.HttpResponse: Defaults to rendering
                ``django_cradmin/invalid_roleid.django.html``.
        """
        return render(self.request, 'django_cradmin/invalid_roleid.django.html', {
            'roleid': roleid
        })

    def get_role_from_rolequeryset(self, role):
        """
        Returns the given role extracted via the :meth:`.get_rolequeryset`
        queryset.

        Raises ObjectDoesNotExist if the role is not found in the queryset.
        """
        return self.get_rolequeryset().get(pk=role.pk)

    def missing_role_response(self, role):
        """
        This is called whenever someone requests a role that exists but that
        they do not have (where meth:`.get_role_from_rolequeryset` raises ``DoesNotExist``).

        Returns:
            django.http.HttpResponse: Defaults to rendering
                ``django_cradmin/missing_role.django.html``
        """
        return render(self.request, 'django_cradmin/missing_role.django.html', {
            'role': role
        })

    def _get_menu(self):
        if not hasattr(self, '_menu'):
            menuclass = self.menuclass
            self._menu = menuclass(self)
        return self._menu

    def get_menu(self):
        """
        Get the navigation menu for the authenticated user.

        Parameters:
            user (Django User): The user requesting the menu.

        Returns:
            An instance of :obj:`.menuclass` by default, but you can
            override this method to determine/create the menu dynamically.

        See Also:
            :class:`django_cradmin.menu.BaseCrAdminMenu`.
        """
        return self._get_menu()

    def reverse_url(self, appname, viewname, args=None, kwargs=None, roleid=None):
        """
        Reverse an URL within this cradmin instance.

        The advantage over using :func:`django.core.urlresolvers.reverse`
        is that you do not need to hardcode the id of the cradmin instance,
        and that the ``roleid`` is automatically added to args or kwargs
        (depending on which one you use to pass arguments to the url).
        """
        if roleid is None:
            roleid = self.get_roleid(self.request.cradmin_role)
        return reverse_cradmin_url(
            instanceid=self.id,
            appname=appname, viewname=viewname, roleid=roleid,
            args=args, kwargs=kwargs)

    def appindex_url(self, appname, roleid=None):
        """
        Reverse the url of the landing page for the given app.

        The landing page is the view named :obj:`django_cradmin.crapp.INDEXVIEW_NAME`.
        """
        return self.reverse_url(appname, viewname=crapp.INDEXVIEW_NAME, roleid=roleid)

    def rolefrontpage_url(self, roleid):
        """
        Returns the URL that the user should be redirected to
        after selecting a role.
        """
        return self.appindex_url(self.rolefrontpage_appname, roleid=roleid)

    def roleselectview_url(self):
        """
        Return the URL of the roleselect view.

        See:
            :meth:`.get_roleselect_view`.
        """
        return reverse('{}-roleselect'.format(self.id))

    @classmethod
    def get_roleselect_view(cls):
        """
        The view for selecting role.

        Defaults to::

            from django_cradmin.views import roleselect
            from django.contrib.auth.decorators import login_required
            return login_required(roleselect.RoleSelectView.as_view())

        If you want to provide your own role select view, you can simply implement
        it here. Another option is to extend RoleSelectView and override the ``template_name``.

        Note:

            The name of the URL for this view is ``<cradmin instance id>-roleselect``,
            where ``<cradmin instance id>`` is :obj:`.id`. You can reverse the URL of
            this view with :meth:`.roleselectview_url`.
        """
        return login_required(roleselect.RoleSelectView.as_view())

    @classmethod
    def matches_urlpath(cls, urlpath):
        """
        If you have more than one BaseCrAdminInstance in a single Django
        instance, you have to implement this classmethod on each of the
        instances to be able to lookup/detect the _current_ BaseCrAdminInstance.

        Parameters:
            urlpath: The url path (HttpRequest.path) to match against.

        Returns:
            ``True`` if the given urlpath is for a view within this CrAdmin instance.
        """
        raise NotImplementedError()

    @classmethod
    def get_apps(cls):
        """
        See :obj:`.apps`. Defaults to returning :obj:`.apps`, but can be overridden.
        """
        return cls.apps

    @classmethod
    def _get_app_urls(cls):
        urls = []
        for appname, appclass in cls.get_apps():
            appurlpatterns = appclass.build_urls(cls.id, appname)
            urls.append(url(r'^(?P<roleid>{})/{}/'.format(
                cls.roleid_regex, appname),
                include(appurlpatterns)))
        return urls

    @classmethod
    def urls(cls):
        """
        Get the url patterns for the cradmin instance.
        """
        cradmin_instance_registry.add(cls)
        return patterns(
            '',
            url('^$', cls.get_roleselect_view(),
                name='{}-roleselect'.format(cls.id)),
            *cls._get_app_urls())



import urllib
import urlparse
from django.utils.translation import ugettext_lazy as _
from django.views.generic import CreateView as DjangoCreateView

from django_cradmin.crispylayouts import PrimarySubmit
from django_cradmin.crispylayouts import DefaultSubmit
from .crudbase import CreateUpdateViewMixin


class CreateView(CreateUpdateViewMixin, DjangoCreateView):
    template_name = 'django_cradmin/viewhelpers/create.django.html'

    #: If this is ``True`` (default), we go into foreignkey select mode
    #: if ``foreignkey_select_mode=1`` is in the querystring.
    #:
    #: Foreignkey select mode:
    #:
    #: - replaces the normal save buttons with a ``submit-use`` button.
    #: - removes the menu (sets the ``cradmin_hide_menu`` template context variable to ``True``).
    #: - adds a ``foreignkey_selected_value=<selected pk>`` to the querystring of the success url.
    allow_foreignkey_select = True

    submit_use_label = _('Create and select')
    submit_save_label = _('Create')
    submit_save_and_continue_edititing_label = _('Create and continue editing')

    def get_buttons(self):
        if self._foreignkey_select_mode():
            buttons = [
                PrimarySubmit('submit-use', _('Create and select')),
            ]
        else:
            buttons = [
                PrimarySubmit('submit-save', self.submit_save_label),
                DefaultSubmit('submit-save-and-continue-editing', self.submit_save_and_continue_edititing_label),
            ]
        preview_url = self.get_preview_url()
        if preview_url:
            buttons.append(DefaultSubmit('submit-preview', _('Preview')))
        return buttons

    def get_default_save_success_url(self):
        url = super(CreateView, self).get_default_save_success_url()
        if self._foreignkey_select_mode():
            url = urllib.unquote_plus(url)
            urllist = list(urlparse.urlsplit(url))
            querystring = urllist[3]
            querydict = urlparse.parse_qs(querystring)
            querydict['foreignkey_selected_value'] = [unicode(self.object.pk)]
            urllist[3] = urllib.urlencode(querydict, doseq=True)
            url = urlparse.urlunsplit(urllist)
        return url

    def get_formhelper(self):
        helper = super(CreateView, self).get_formhelper()
        helper.form_id = 'django_cradmin_createform'
        return helper

    def _foreignkey_select_mode(self):
        return self.allow_foreignkey_select and self.request.GET.get('foreignkey_select_mode') == '1'

    def get_context_data(self, **kwargs):
        context = super(CreateView, self).get_context_data(**kwargs)
        context['cradmin_hide_menu'] = self._foreignkey_select_mode()
        return context

    def get_success_message(self, object):
        return _('Created "%(object)s".') % {'object': object}



from PIL import Image, ImageDraw
from cStringIO import StringIO


def create_image(width, height, filetype='PNG'):
    """
    Create an image of the given ``width``, ``height`` and optional ``filetype``.

    Returns:
        A bytestring with the generated image.
    """
    im = Image.new('RGB', (width, height), color='#191919')
    ImageDraw.Draw(im)
    out = StringIO()
    im.save(out, filetype)
    return out.getvalue()



import os
from sorl.thumbnail import get_thumbnail


def request_thumbnail(
        image, width=None, height=None, image_format='auto', colorspace='RGB',
        upscale=True, quality=95,
        scalemethod='contain'):
    """
    Thin wrapper around sorl-thumbnail (``sorl.thumbnail.get_thumbnail``).

    Requires that you have sorl-thumbnail installed.

    Parameters:
        image: A django ImageField object.
        width:
            The maximum width of the resulting image.
            How this is handled depends on the ``scalemethod``.
            Can be ``None`` if ``height`` is specified.
        height:
            The maximum height of the resulting image.
            How this is handled depends on the ``scalemethod``.
            Can be ``None`` if ``width`` is specified.
        image_format:
            Must be one of "auto", "JPEG" or "PNG".
        upscale:
            Upscale is a boolean and controls if the image can be upscaled or not.
            For example if your source is 100x100 and you request a thumbnail of size
            ``200x200`` and upscale is False this will return a thumbnail of size ``100x100``.
            If upscale was True this would result in a thumbnail size ``200x200`` (upscaled).
            The default value is ``True``.
        quality:
            Quality is a value between 0-100 and controls the thumbnail write quality.
            Default value is 95.
        colorspace:
            This controls the resulting thumbnails color space, valid values
            are: ``'RGB'`` and ``'GRAY'``. Default value is ``'RGB'``.
        scalemethod:
            How do we scale the image. Possible values:

            - ``contain``: Scale the image down so the entire image fits within
              the ``width`` and ``height``.
            - ``cover``: Scale down and crop the image to make it fill the
              ``width`` and ``height`` box.

            The default is ``contain``.
    """
    if not width and not height:
        raise ValueError('At least one of width and height must be specified')
    width = width or ''
    height = height or ''
    size = '{}x{}'.format(width, height)

    if scalemethod == 'contain':
        crop = None
    elif scalemethod == 'cover':
        crop = 'center'
    else:
        raise ValueError('{!r} is an invalid value for scalemethod. Valid values: "contain", "cover".')

    if image_format == 'auto':
        if hasattr(image, 'name'):
            path = image.name
        else:
            path = image
        name, extension = os.path.splitext(path)
        if extension in ('.png', '.gif'):
            detected_format = 'PNG'
        else:
            detected_format = 'JPEG'
    elif image_format not in ('JPEG', 'PNG'):
        raise ValueError('{!r} is an invalid value for image_format. Valid values: "auto", "JPEG", "PNG".')
    else:
        detected_format = image_format
    return get_thumbnail(
        image, size,
        upscale=upscale,
        quality=quality,
        crop=crop,
        format=detected_format,
        colorspace=colorspace)



from django.core.exceptions import ValidationError
from django.db import models


class SortableManagerBase(models.Manager):
    """
    Manager for :class:`.SortableBase`.
    """
    parent_attribute = None

    def _get_filtered_by_parentobject_queryset(self, parentobject):
        filter_kwargs = {self.parent_attribute: parentobject}
        return self.get_queryset().filter(**filter_kwargs).order_by('sort_index')

    def _get_siblings_queryset(self, item):
        """
        Get the queryset for all the items with the same parent as the given
        item.
        """
        parentobject = getattr(item, self.parent_attribute)
        return self._get_filtered_by_parentobject_queryset(parentobject)

    def _get_last_sortindex_within_parentobject(self, item):
        lastitem = self._get_siblings_queryset(item).last()
        if lastitem:
            return lastitem.sort_index + 1
        else:
            return 1

    def set_newitem_sort_index_to_last(self, item):
        """
        Sets ``item.sort_index`` to the sort_index of the last item
        in the parent + 1. Does not save.

        ONLY USE THIS FOR NEWLY CREATED ITEMS.
        """
        if item.sort_index is not None or item.pk is not None:
            raise ValueError('set_newitem_sort_index_to_last should only be used '
                             'when creating new items. sort_index or pk is something '
                             'other than None, which indicates that this is not a new '
                             'item - thus the ValueError.')
        item.sort_index = self._get_last_sortindex_within_parentobject(item)

    def sort_before(self, item, sort_before_id):
        """
        Sort a given item before the item with id `sort_before_id`,
        or last if `sort_before_id` is ``None``.

        Fetches all items in the same container, and makes changes in the ordering.
        Only the required updates are made.
        """
        queryset = self._get_siblings_queryset(item)
        items = queryset.all()

        # Get the current position for the item to sort before
        before_item_found_index = 0
        # Get the current position for the item to move
        item_current_index = 0
        # Loop over all items to find `before_item_found_index` and `item_current_index`,
        # and to set the new value of sort_index on the item we are moving
        for i in range(0, len(items), 1):
            cur_item = items[i]
            # sorting last is a special case
            if sort_before_id is None and i == len(items) - 1:
                item.sort_index = cur_item.sort_index
                item.save()
                before_item_found_index = i + 1
            # set new sort_order for item
            elif cur_item.id == sort_before_id:
                item.sort_index = cur_item.sort_index
                item.save()
                before_item_found_index = i
            # set the current item index
            if cur_item.id == item.id:
                item_current_index = i

        # first move if item should be last - the special case
        if sort_before_id is None:
            add_on_index_for_items = []
            for itm in items[item_current_index:before_item_found_index]:
                if itm != item:
                    add_on_index_for_items.append(itm.id)
            queryset.filter(id__in=add_on_index_for_items).update(sort_index=models.F('sort_index') - 1)

        elif item_current_index > before_item_found_index:
            # Moving item up
            add_on_index_for_items = []
            for itm in items[before_item_found_index:item_current_index]:
                add_on_index_for_items.append(itm.id)
            queryset.filter(id__in=add_on_index_for_items).update(sort_index=models.F('sort_index') + 1)

        elif item_current_index < before_item_found_index:
            # Moving item down
            subtract_on_index_for_items = []
            for itm in items[item_current_index:before_item_found_index]:
                subtract_on_index_for_items.append(itm.id)
            queryset.filter(id__in=subtract_on_index_for_items).update(sort_index=models.F('sort_index') - 1)

    def sort_last(self, item):
        """
        Just a shortcut for::

            self.sort_before(item, sort_before_id=None)
        """
        self.sort_before(item, sort_before_id=None)


def validate_sort_index(value):
    if value < 1:
        raise ValidationError(u'Sort index must be 1 or higher.')


class SortableBase(models.Model):
    """
    Used with :class:`.SortableManagerBase` to make models sortable.
    """

    #: Sort index - ``1`` or higher.
    sort_index = models.PositiveIntegerField(
        blank=False, null=False,
        verbose_name='index',
        validators=[validate_sort_index]
    )

    class Meta:
        abstract = True



from collections import OrderedDict
import json
import re
import logging
from xml.sax.saxutils import quoteattr
from django import forms
from django.shortcuts import get_object_or_404
from django.template import defaultfilters
from django.template.defaultfilters import truncatechars
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.views.generic import ListView
from django.db import models


logger = logging.getLogger(__name__)


class Column(object):

    #: The model field rendered in this column. It is not required,
    #: but if you do not, you have to override :meth:`.get_header`
    #: and :meth:`.render_value`.
    modelfield = None

    #: A field to use for ordering of this model. Can also be a join, like ``company__name``.
    #: If this is not specified, :obj:`.modelfield` is used. If none of them are used,
    #: or if you want to sort using multiple fields, you must override :meth:`.get_orderby_args`
    #: and :meth:`.is_sortable`.
    orderingfield = None

    #: The template used to render a cell in the Column.
    template_name = None

    #: The column width. See :meth:`.get_column_width`.
    column_width = None

    #: The name of the template context object.
    #: The object is always available as ``object``, but you
    #: can override this to make it available by another name.
    context_object_name = None

    #: The name of the template context variable containing the return value from
    #: meth:`.get_value`.
    #: This is always available as ``value``, but you
    #: can override this to make it available by another name.
    context_value_name = None

    #: List of css classes to add to the header cell in this column.
    #: See :meth:`.get_headercell_css_classes`.
    headercell_css_classes = []

    #: List of css classes to add to all the non-header cells in this column.
    #: See :meth:`.get_normalcells_css_classes`.
    normalcells_css_classes = []

    #: List of css classes to add to both normal cells and the header cell in this column.
    #: See :meth:`.get_allcells_css_classes`.
    allcells_css_classes = []

    def __init__(self, view, columnindex):
        self.view = view
        self.columnindex = columnindex

    def get_header(self):
        """
        Get the header of the column. Defaults to the ``verbose_name``
        of the :obj:`.modelfield`.
        """
        if self.modelfield:
            field = self.view.model._meta.get_field(self.modelfield)
            return field.verbose_name
        else:
            raise NotImplementedError()

    def get_column_width(self):
        """
        Returns the column width. Setting this is generally not recommended, but
        sometimes you need exact scaling of the column.

        Example legal values: ``"100px", "20%"``. Defaults to :obj:`.column_width`.
        """
        return self.column_width

    def get_headercell_css_style(self):
        """
        Get the css styles of the header cell of the column.
        Defaults to setting :meth:`.get_column_width` as a css style.

        You normally want to avoid setting styles with this and use
        :meth:`.get_headercell_css_class_string` instead, but this is provided for
        those cases where setting the style attribute is the only decent solution.
        """
        column_width = self.get_column_width()
        if column_width:
            return 'width: {}'.format(column_width)
        else:
            return ''

    def get_headercell_css_classes(self):
        """
        Get the css classes of the header cell in this column.

        Returns:
            A list of css classes. Defaults to :obj:`.headercell_css_classes`.
        """
        return self.headercell_css_classes

    def get_headercell_css_class_string(self):
        """
        Get the css class of the header cell of the column.
        Defaults to setting the ``objecttableview-sortable-header``
        class if :meth:`.is_sortable`.
        """
        css_classes = self.get_headercell_css_classes() + self.get_allcells_css_classes()
        if self.is_sortable():
            css_classes.append('objecttableview-sortable-header')
        return ' '.join(css_classes)

    def get_normalcells_css_classes(self):
        """
        Get the css classes of all the normal (non-header) cells in this
        column.

        Returns:
            A list of css classes. Defaults to :obj:`.normalcells_css_classes`.
        """
        return self.normalcells_css_classes

    def get_normalcell_css_class_string(self):
        """
        Get the css class of the header cell of the column.
        You normally do not override this. Override
        :meth:`.get_normalcells_css_classes` instead.
        """
        css_classes = self.get_normalcells_css_classes() + self.get_allcells_css_classes()
        return ' '.join(css_classes)

    def get_allcells_css_classes(self):
        """
        Get css classes that should be added to all cells in this
        column, including the header cell.

        Returns:
            A list of css classes. Defaults to :obj:`.allcells_css_classes`.
        """
        return self.allcells_css_classes

    def reverse_appurl(self, name, args=[], kwargs={}):
        return self.view.request.cradmin_app.reverse_appurl(name, args=args, kwargs=kwargs)

    def render_value(self, obj):
        if self.modelfield:
            return getattr(obj, self.modelfield)
        else:
            raise NotImplementedError()

    def get_context_data(self, obj):
        """
        Get context data for rendering the cell (see :meth:`.render_cell_content`.
        """
        value = self.render_value(obj)
        context = {
            'value': value,
            'object': obj
        }
        if self.context_object_name:
            context[self.context_object_name] = obj
        if self.context_value_name:
            context[self.context_value_name] = value
        return context

    def render_cell_content(self, obj):
        """
        Render the cell using the template specifed in :obj:`.template_name`.
        """
        return render_to_string(self.template_name, self.get_context_data(obj=obj))

    def get_flip_ordering_url(self):
        """
        Used in the template to get the URL when a user clicks on the header.
        Should not be overridden.
        """
        return self.view._get_flip_ordering_url_for_column(self.columnindex)

    def get_remove_ordering_url(self):
        """
        Used in the template to get the URL when a user clicks on the remove ordering button in the header.
        Should not be overridden.
        """
        return self.view._get_remove_ordering_url_for_column(self.columnindex)

    @property
    def orderinginfo(self):
        if not hasattr(self, '_orderinginfo'):
            if self.is_sortable():
                self._orderinginfo = self.view._get_orderinginfo_for_column(self.columnindex)
            else:
                self._orderinginfo = None
        return self._orderinginfo

    def is_ordered(self):
        return self.orderinginfo and self.orderinginfo.order_ascending is not None

    def is_sortable(self):
        """
        Specifies if this column is sortable.

        Defaults to ``True`` if :obj:`.modelfield` or :obj:`.orderingfield`
        is defined.
        """
        return bool(self.modelfield) or bool(self.orderingfield)

    def get_orderby_args(self, order_ascending):
        """
        Get the a list of ``QuerySet.order_by`` arguments that should be applied
        when the user requests the given ``sortorder`` for this column.

        If you set :obj:`.orderingfield`, the value is generated from that. If
        :obj:`.orderingfield` is not defined, we fall back on :obj:`.modelfield`.

        Parameters:
            order_ascending (bool): ``True`` if we should return the orderby arguments
                for ascending ordering, and ``False`` for descending ordering.
        """
        sortprefix = ''
        if not order_ascending:
            sortprefix = '-'
        orderingfield = self.orderingfield or self.modelfield
        if orderingfield:
            return ['{}{}'.format(sortprefix, orderingfield)]
        else:
            raise NotImplementedError('You must override get_orderby_args, set orderingfield or set modelfield.')

    def get_default_order_is_ascending(self):
        """
        Determine if the default ordering for the field shown in this column is
        ascending.

        Returns:
            ``True`` if the default ordering is ascending.
            ``False`` if the default ordering is descending.
            ``None`` if no default ordering is configured for this column.
        """
        orderingfield = self.orderingfield or self.modelfield
        if orderingfield:
            default_ordering = self.view.model._meta.ordering
            descending_orderingfield = '-{}'.format(orderingfield)
            if orderingfield in default_ordering:
                return True
            elif descending_orderingfield in default_ordering:
                return False
            else:
                return None
        else:
            raise NotImplementedError(
                'You must return False from is_sortable(), override '
                'get_default_order_is_ascending(), set orderingfield or set modelfield.')


class PlainTextColumn(Column):
    template_name = 'django_cradmin/viewhelpers/objecttable/plaintextcolumn-cell.django.html'


class DatetimeColumn(PlainTextColumn):
    datetime_format = 'SHORT_DATETIME_FORMAT'

    def render_value(self, obj):
        value = super(DatetimeColumn, self).render_value(obj)
        if value is None:
            return None
        else:
            return defaultfilters.date(value, self.datetime_format)


class TruncatecharsPlainTextColumn(PlainTextColumn):
    """
    A variant of :class:`.PlainTextColumn` that truncates the text down to
    a configurable length. The length is configured in :obj:`.maxlength`.
    """

    #: Maximum length of the value displayed in the column.
    maxlength = 30

    def render_value(self, obj):
        value = super(TruncatecharsPlainTextColumn, self).render_value(obj)
        if value is None:
            return None
        else:
            return truncatechars(value, self.maxlength)


class SingleActionColumn(Column):
    template_name = 'django_cradmin/viewhelpers/objecttable/singleactioncolumn-cell.django.html'

    def __init__(self, **kwargs):
        super(SingleActionColumn, self).__init__(**kwargs)

    def get_actionurl(self, obj):
        raise NotImplementedError()

    def get_context_data(self, obj):
        context = super(SingleActionColumn, self).get_context_data(obj=obj)
        context['action_url'] = self.get_actionurl(obj)
        return context


class ImagePreviewColumn(Column):
    template_name = 'django_cradmin/viewhelpers/objecttable/imagepreviewcolumn-cell.django.html'

    #: See :meth:`.get_preview_format`
    preview_format = 'auto'

    #: See :meth:`.get_preview_width`
    preview_width = 100

    #: See :meth:`.get_preview_height`
    preview_height = 70

    def is_sortable(self):
        return False

    def get_preview_width(self):
        """
        Returns the width of the preview thumbnail. Can be overridden.
        Defaults to :obj:`.preview_width`, so you can just override that
        class variable instead of this method.
        """
        return self.preview_width

    def get_preview_height(self):
        """
        Returns the height of the preview thumbnail. Can be overridden.
        Defaults to :obj:`.preview_height`, so you can just override that
        class variable instead of this method.
        """
        return self.preview_height

    def get_preview_format(self):
        """
        Returns the format of the preview thumbnail. Can be overridden.
        Defaults to :obj:`.preview_format`, so you can just override that
        class variable instead of this method.
        """
        return self.preview_format

    def get_column_width(self):
        return u'{}px'.format(self.get_preview_width())

    def get_context_data(self, obj):
        context = super(ImagePreviewColumn, self).get_context_data(obj=obj)
        image_path = None
        imagefieldfile = self.render_value(obj)
        if imagefieldfile:
            image_path = imagefieldfile.name
        context.update({
            'image_path': image_path,
            'preview_width': self.get_preview_width(),
            'preview_height': self.get_preview_height(),
            'preview_format': self.get_preview_format()
        })
        return context


class MultiActionColumn(Column):
    template_name = 'django_cradmin/viewhelpers/objecttable/multiactioncolumn-cell.django.html'

    def __init__(self, **kwargs):
        super(MultiActionColumn, self).__init__(**kwargs)

    def get_buttons(self, obj):
        """
        Get an iterable over the buttons/actions for the given ``obj``.

        Returns:
            An iterable yielding :class:`.Button` objects.
        """
        raise NotImplementedError()

    def get_context_data(self, obj):
        context = super(MultiActionColumn, self).get_context_data(obj=obj)
        context['buttons'] = self.get_buttons(obj)
        return context


class UseThisActionColumn(Column):
    template_name = 'django_cradmin/viewhelpers/objecttable/usethisactioncolumn-cell.django.html'

    def __init__(self, **kwargs):
        super(UseThisActionColumn, self).__init__(**kwargs)

    def get_buttons(self, obj):
        """
        Get an iterable over the buttons/actions for the given ``obj``.

        Returns:
            An iterable yielding :class:`.Button` objects.
        """
        raise NotImplementedError()

    def get_context_data(self, obj):
        context = super(UseThisActionColumn, self).get_context_data(obj=obj)
        context['buttons'] = self.get_buttons(obj)
        return context


class Button(object):
    """
    A pythonic interface for creating a HTML link styled as a button.

    Used by:

    - :meth:`.ObjectTableView.get_buttons`.
    - :meth:`.MultiActionColumn.get_buttons`
    """
    template_name = 'django_cradmin/viewhelpers/objecttable/button.django.html'

    def __init__(self, label, url='#', buttonclass='default', icon=None):
        """
        Parameters:
            label (unicode): The label of the button.
            url (unicode): The url/href attribute of the button.
            icon (unicode): An icon to show alongside the label. Example: ``fa-thumbs-up``.
            buttonclass (unicode): The bootstrap css class suffix of the button (default|primany|success|danger).
        """
        self.label = label
        self.url = url
        self.icon = icon
        self.buttonclass = buttonclass

    def get_attributes(self):
        """
        Returns a dict of custom attributes to add to the button.
        They are automatically escaped before they are added to the button.
        """
        return {}

    def _iter_attributes(self):
        for key, value in self.get_attributes().iteritems():
            attrname = u'data-{}'.format(key)
            attrvalue = quoteattr(value)
            yield u'{}={}'.format(attrname, attrvalue)

    def render(self):
        return render_to_string(self.template_name, {
            'label': self.label,
            'url': self.url,
            'icon': self.icon,
            'buttonclass': self.buttonclass,
            'attributes': self._iter_attributes(),
        })


class PagePreviewButton(Button):
    """
    A button variant that uses the ``django-cradmin-page-preview-open-on-click``
    AngularJS directive to open a preview in an overlay containing an IFRAME.
    Works just like a regular button. The only difference is that the url is
    opened in the IFRAME in the overlay instead of in the current window.

    For this to work, you need to set :obj:`.ObjectTableView.enable_previews` to ``True``
    (or override :meth:`.ObjectTableView.get_enable_previews`).
    """
    def get_attributes(self):
        return {
            'django-cradmin-page-preview-open-on-click': self.url
        }


class UseThisButton(Button):
    """
    Button for :class:`.UseThisActionColumn`.
    """
    def __init__(self, view, label, obj, buttonclass='default', icon=None):
        self.view = view
        self.obj = obj
        super(UseThisButton, self).__init__(label=label, buttonclass=buttonclass, icon=icon)

    def get_attributes(self):
        attributes = {
            'django-cradmin-use-this': json.dumps({
                'value': self.obj.pk,
                'fieldid': self.view.request.GET['foreignkey_select_fieldid'],
                'preview': self.view.make_foreignkey_preview_for(self.obj)
            })
        }
        return attributes


class ForeignKeySelectButton(Button):
    """
    Used to make buttons that lead to views that handle foreign key selection.

    It is just like a normal :class:`.Button`, except that it requires a request
    object, and uses that to set success_url to the current URL.
    """
    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request')
        super(ForeignKeySelectButton, self).__init__(*args, **kwargs)
        self.url = '{}?{}'.format(self.url, urlencode({
            'foreignkey_select_mode': '1',
            'success_url': request.get_full_path()
        }))


class MultiSelectAction(object):
    """
    Used to define multiselect actions for
    :meth:`.ObjectTableView.get_multiselect_actions`.
    """
    def __init__(self, label, url):
        self.label = label
        self.url = url

    def serialize(self):
        return {
            'label': unicode(self.label),
            'url': self.url
        }


class ColumnOrderingInfo(object):
    @classmethod
    def from_orderingstringentry(cls, orderingstringindex, orderingstringentry):
        order_ascending = orderingstringentry.startswith('a')
        columnindex = int(orderingstringentry[1:])
        return cls(
            orderingstringindex=orderingstringindex,
            columnindex=columnindex,
            order_ascending=order_ascending)

    @classmethod
    def create_orderingstringentry(cls, columnindex, order_ascending):
        if order_ascending:
            prefix = 'a'
        else:
            prefix = 'd'
        return '{}{}'.format(prefix, columnindex)

    def __init__(self, orderingstringindex, columnindex, order_ascending):
        self.orderingstringindex = orderingstringindex
        self.columnindex = columnindex
        self.order_ascending = order_ascending

    def to_orderingstringentry(self):
        return self.__class__.create_orderingstringentry(self.columnindex, self.order_ascending)

    def __str__(self):
        return '{}:{}'.format(self.orderingstringindex, self.to_orderingstringentry())


class OrderingStringParser(object):
    """
    Parses an ordering string (the ordering querystring argument for objecttable).

    Parameters:
        orderingstring (str):
            A ``.``-separated list of the character ``a`` or ``d`` followed by
            a 0-based column index. The character described the order we want
            to sort the column at the given index (a: ascending, d: descending).

            The order of the entries in the comma separated list has significance.
            For the sortstring ``a3,a1,a2``, we will sort by column 3 first,
            then by column 1 and then by column 2 (all ascending).

            Note that the OrderingStringParser only describes what columns to
            order and in what direction, not by what fields each column is ordered.
    """
    def __init__(self, orderingstring):
        self.orderingstring = orderingstring
        self.orderingdict = OrderedDict()
        if self.orderingstring:
            if re.match('^([ad]\d+\.)*([ad]\d+)$', self.orderingstring):
                for orderingstringindex, orderingstringentry in enumerate(self.orderingstring.split('.')):
                    orderinginfo = ColumnOrderingInfo.from_orderingstringentry(
                        orderingstringindex=orderingstringindex,
                        orderingstringentry=orderingstringentry)
                    self.orderingdict[orderinginfo.columnindex] = orderinginfo
            else:
                logger.debug('Invalid value for ordering: %r', self.orderingstring)

    def remove_column(self, columnindex):
        """
        Return an orderingstring on the same format as the one parsed by this class
        with the given ``columnindex`` removed.
        """
        out = []
        for index, orderinginfo in self.orderingdict.iteritems():
            if index != columnindex:
                out.append(orderinginfo.to_orderingstringentry())
        return '.'.join(out)

    def flip_column(self, columnindex):
        """
        Return an orderingstring on the same format as the one parsed by this class
        with the ordering direction of the given ``columnindex`` flipped/reversed.

        If the given column is not ordered, we add it last with ascending ordering.
        """
        out = []
        found = False
        for index, orderinginfo in self.orderingdict.iteritems():
            if index == columnindex:
                found = True
                order_ascending = not orderinginfo.order_ascending
                out.append(ColumnOrderingInfo.create_orderingstringentry(
                    columnindex=columnindex, order_ascending=order_ascending))
            else:
                out.append(orderinginfo.to_orderingstringentry())
        if not found:
            out.append(ColumnOrderingInfo.create_orderingstringentry(
                columnindex=columnindex, order_ascending=True))
        return '.'.join(out)

    def get(self, columnindex):
        return self.orderingdict.get(columnindex, None)

    def __len__(self):
        return len(self.orderingdict)


class SearchForm(forms.Form):
    search = forms.CharField(required=True)


class ObjectTableView(ListView):
    """
    A view inspired by the changelist in ``django.contrib.admin``.

    Lets say you use company as the role in your admin site, and
    you want each company to have a repository of documents that
    they can offer for download. To make a list of all uploaded
    documents, you would do something like this::

        from django_cradmin.viewhelpers.objecttable import ObjectTableView
        from myapp.models import Document

        class DocumentListView(ObjectTableView):
            model = Document

            def get_queryset_for_role(self, company):
                return self.model.objects.filter(
                    company=company)

    """
    #: Paginate by this number of items. You can safely override this.
    paginate_by = 30

    #: The template used to render the view. You can override this,
    #: but if you override it, you should extend the default template
    #: (django_cradmin/viewhelpers/objecttable/objecttable.django.html).
    template_name = 'django_cradmin/viewhelpers/objecttable/objecttable.django.html'

    #: Set this to ``True`` to make the template not render the menu.
    #: Very useful when creating foreign-key select views, and other views
    #: where you do not want your users to accidentally click out of the
    #: current view.
    hide_menu = False

    #: The model class to list objects for. You do not have to specify
    #: this, but if you do not specify this, you have to override
    #: :meth:`.get_pagetitle`.
    model = None

    #: Defines the columns in the table. See :meth:`.get_columns`.
    columns = []

    #: The fields to search.
    #: Setting this enables search. If you implement a custom
    #: :meth:`.filter_search` method that does not make use of
    #: ``searchfields``, you can override :meth:`.enable_search`
    #: instead of setting this to some fake value.
    searchfields = []

    #: The default implementation of :meth:`.filter_search` uses this comparator
    #: when searching the given :obj:`.searchfields`.
    search_comparator = 'icontains'

    #: The search placeholder text. See :meth:`.get_enable_previews`. Defaults to "Search...".
    search_placeholder_text = _('Search...')

    #: Enable previews? See :meth:`.get_enable_previews`. Defaults to ``False``.
    enable_previews = False

    def get_search_placeholder_text(self):
        """
        Returns the search placeholder text. Defaults to :obj:`.search_placeholder_text`.

        Only used if search is enabled (see :obj:`.searchfields`).
        """
        return self.search_placeholder_text

    def get_enable_previews(self):
        """
        If this returns ``True``, we enable previews. When previews are
        enabled, you can add :class:`.PagePreviewButton` to columns
        to show previews.

        Defaults to :obj:`.enable_previews`.
        """
        return self.enable_previews

    def get_multiselect_actions(self):
        """
        Multiselect actions. If this returns a non-empty iterator, each row
        has a cell with a select box in the first column. Selecting at least
        one of the rows will reveal a dropdown menu with all the available
        actions.

        Returns:
            An iterator (list, tuple, ...) yielding :class:`.MultiSelectAction`
            objects. Returns an empty list by default.
        """
        return []

    def get_columns(self):
        """
        Get the columns in the table. Defaults to :obj:`.columns`.

        A list where each entry can be the name of any field on the
        :class:`model`, or a subclass of :class:`.Column`.
        """
        return self.columns

    def get_buttons(self):
        """
        Override this to add some buttons/global actions for this table.

        Example::

            def get_buttons(self):
                app = self.request.cradmin_app
                return [
                    objecttable.Button('Create', url=app.reverse_appurl('create')),
                    objecttable.Button('Delete all', url=app.reverse_appurl('deleteall')),
                ]
        """
        return []

    def get_queryset_for_role(self, role):
        """
        Get a queryset with all objects of :obj:`.model`  that
        the current role can access.
        """
        raise NotImplementedError()

    def enable_search(self):
        """
        Enable search?

        The default implementation returns ``True`` if :obj:`.searchfields` is
        not empty.
        """
        return len(self.searchfields) > 0

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role(self.request.cradmin_role)
        orderby = []
        for columnobject, orderinginfo in self.__iter_columnobjects_with_orderinginfo():
            orderby.extend(columnobject.get_orderby_args(orderinginfo.order_ascending))
        if orderby:
            queryset = queryset.order_by(*orderby)
        if self.enable_search():
            searchform = SearchForm(self.request.GET)
            self.current_search = ''
            if searchform.is_valid():
                self.current_search = searchform.cleaned_data['search']
                queryset = self.filter_search(searchstring=self.current_search, queryset=queryset)
        return queryset

    def _get_columnobjects(self):
        """
        Build the :obj:`.columns` list into a list of :class:`Column` objects.
        """
        if not hasattr(self, '__columns'):
            self.__columns = []
            for columnindex, columnclass in enumerate(self.columns):
                if isinstance(columnclass, basestring):
                    columnclass = type('SimpleColumn', (PlainTextColumn,), dict(modelfield=columnclass))
                self.__columns.append(columnclass(view=self, columnindex=columnindex))
        return self.__columns

    def __create_row(self, obj):
        return {
            'object': obj,
            'cells': [(column, column.render_cell_content(obj)) for column in self._get_columnobjects()]
        }

    def __iter_table(self, object_list):
        for obj in object_list:
            yield self.__create_row(obj)

    def get_pagetitle(self):
        """
        Get the page title/heading.

        Defaults to the ``verbose_name_plural`` of the :obj:`.model`.
        """
        return self.model._meta.verbose_name_plural

    def _get_search_hidden_fields(self):
        for key, value in self.request.GET.iteritems():
            if key != 'search':
                yield key, value

    def _get_pager_extra_querystring(self):
        querystring = self.request.GET.copy()
        if 'page' in querystring:
            del querystring['page']
        if querystring:
            return urlencode(querystring)
        else:
            return ''

    def make_foreignkey_preview_for(self, obj):
        return unicode(obj)

    def _get_use_this_hidden_attribute(self):
        pk = self.request.GET['foreignkey_selected_value']
        obj = get_object_or_404(self.get_queryset_for_role(self.request.cradmin_role), pk=pk)
        data = json.dumps({
            'value': obj.pk,
            'fieldid': self.request.GET['foreignkey_select_fieldid'],
            'preview': self.make_foreignkey_preview_for(obj)
        })
        return quoteattr(data)

    def get_context_data(self, **kwargs):
        context = super(ObjectTableView, self).get_context_data(**kwargs)
        object_list = context['object_list']
        multiselect_actions = self.get_multiselect_actions()
        if multiselect_actions:
            context['multiselect_actions'] = json.dumps(
                [action.serialize() for action in multiselect_actions])
        context['pagetitle'] = self.get_pagetitle()
        context['columns'] = self._get_columnobjects()
        context['table'] = list(self.__iter_table(object_list))
        context['buttons'] = self.get_buttons()
        context['enable_search'] = self.enable_search()
        context['enable_previews'] = self.get_enable_previews()
        context['cradmin_hide_menu'] = self.hide_menu
        if self.enable_search():
            context['current_search'] = self.current_search
            context['search_hidden_fields'] = self._get_search_hidden_fields()
        context['pager_extra_querystring'] = self._get_pager_extra_querystring()
        context['multicolumn_ordering'] = len(self.__parse_orderingstring()) > 1

        # Handle foreignkey selection
        if 'foreignkey_selected_value' in self.request.GET:
            context['use_this_hidden_attribute'] = self._get_use_this_hidden_attribute()

        return context

    def __create_orderingstring_from_default_ordering(self):
        orderinglist = []
        for columnindex, columnobject in enumerate(self._get_columnobjects()):
            if columnobject.is_sortable():
                order_ascending = columnobject.get_default_order_is_ascending()
                if order_ascending is not None:
                    orderinglist.append(ColumnOrderingInfo.create_orderingstringentry(
                        columnindex=columnindex,
                        order_ascending=order_ascending
                    ))
        return '.'.join(orderinglist)

    def __parse_orderingstring(self):
        if not hasattr(self, '__orderingstringparser'):
            orderingstring = self.request.GET.get('ordering', None)
            if orderingstring is None:
                orderingstring = self.__create_orderingstring_from_default_ordering()
            self.__orderingstringparser = OrderingStringParser(orderingstring)
        return self.__orderingstringparser

    def _url_with_querystringarg_changed(self, querystringarg, newvalue):
        querystring = self.request.GET.copy()
        querystring[querystringarg] = newvalue
        return '{}?{}'.format(self.request.path, querystring.urlencode())

    def _get_flip_ordering_url_for_column(self, columnindex):
        return self._url_with_querystringarg_changed(
            querystringarg='ordering',
            newvalue=self.__parse_orderingstring().flip_column(columnindex))

    def _get_remove_ordering_url_for_column(self, columnindex):
        return self._url_with_querystringarg_changed(
            querystringarg='ordering',
            newvalue=self.__parse_orderingstring().remove_column(columnindex))

    def _get_orderinginfo_for_column(self, columnindex):
        return self.__parse_orderingstring().get(columnindex)

    def __iter_columnobjects_with_orderinginfo(self):
        columnobjects = self._get_columnobjects()
        for orderinginfo in self.__parse_orderingstring().orderingdict.itervalues():
            try:
                columnobject = columnobjects[orderinginfo.columnindex]
            except IndexError:
                pass
            else:
                yield (columnobject, orderinginfo)

    def filter_search(self, queryset, searchstring):
        """
        Filter the queryset on search.

        The default implementation uses :obj:`.search_comparator` to compare
        earch of the :obj:`.searchfields` with the given ``searchstring``.
        This does not give a very good search experience, but it works
        fine for small data sets.

        If you want to use a search engine, you should make sure your
        search engine can yield a list of ids/primary-keys, and override
        this method with something like this::

             def filter_search(self, queryset, searchstring):
                ids = my_searchengine.search(searchstring)
                return queryset.filter(id__in=ids)

        If you use a custom search engine, you will most likely also want
        to override :meth:`.enable_search` and just return ``True``.
        """
        if self.searchfields:
            query = None
            for fieldname in self.searchfields:
                kwargs = {
                    '{}__{}'.format(fieldname, self.search_comparator): searchstring
                }
                fieldquery = models.Q(**kwargs)
                if query:
                    query |= fieldquery
                else:
                    query = fieldquery
            queryset = queryset.filter(query).distinct()
        return queryset



import json
import os
from django.core.files.base import ContentFile
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import RequestFactory, TestCase
from django_cradmin.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile
from django_cradmin.apps.cradmin_temporaryfileuploadstore.views.temporary_file_upload_api import \
    UploadTemporaryFilesView
from django_cradmin.tests.helpers import create_user


class TestUploadTemporaryFilesView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testuser = create_user('testuser')

    def test_post_single_file(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1')
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        self.assertIsNotNone(responsedata['collectionid'])

        collectionid = responsedata['collectionid']
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.files.count(), 1)
        uploadedfile = collection.files.first()
        self.assertEqual(uploadedfile.filename, 'testfile1.txt')
        self.assertNotIn('testfile1.txt', uploadedfile.file.name)
        self.assertTrue(uploadedfile.file.name.startswith(
            'cradmin_temporaryfileuploadstore/{}/'.format(collection.id)))
        self.assertEqual(uploadedfile.file.read(), 'Test1')

        self.assertEquals(len(responsedata['temporaryfiles']), 1)
        self.assertEquals(responsedata['temporaryfiles'][0]['filename'], 'testfile1.txt')

    def test_post_multiple_files(self):
        request = self.factory.post('/test', {
            'file': [
                SimpleUploadedFile('testfile1.txt', 'Test1'),
                SimpleUploadedFile('testfile2.txt', 'Test2'),
            ]
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        self.assertIsNotNone(responsedata['collectionid'])

        collectionid = responsedata['collectionid']
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.files.count(), 2)
        self.assertEquals(len(responsedata['temporaryfiles']), 2)
        self.assertEquals(responsedata['temporaryfiles'][0]['filename'], 'testfile1.txt')
        self.assertEquals(responsedata['temporaryfiles'][1]['filename'], 'testfile2.txt')

    def test_post_multiple_requests_for_same_collection(self):
        request1 = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1')
        })
        request1.user = self.testuser
        response1 = UploadTemporaryFilesView.as_view()(request1)
        self.assertEquals(response1.status_code, 200)
        response1data = json.loads(response1.content)
        self.assertIsNotNone(response1data['collectionid'])
        collectionid = response1data['collectionid']
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.files.count(), 1)

        request2 = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile2.txt', 'Test2'),
            'collectionid': collectionid
        })
        request2.user = self.testuser
        response2 = UploadTemporaryFilesView.as_view()(request2)
        self.assertEquals(response2.status_code, 200)
        response2data = json.loads(response2.content)
        self.assertEquals(collectionid, response2data['collectionid'])
        self.assertEquals(collection.files.count(), 2)

    def test_post_multiple_files_singlefile_mode(self):
        request1 = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'mode': 'singlefile'
        })
        request1.user = self.testuser
        response1 = UploadTemporaryFilesView.as_view()(request1)
        self.assertEquals(response1.status_code, 200)
        response1data = json.loads(response1.content)
        self.assertIsNotNone(response1data['collectionid'])
        collectionid = response1data['collectionid']
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.files.count(), 1)
        uploadedfile1 = collection.files.first()
        uploadedfile1_physical_file_path = uploadedfile1.file.path
        self.assertTrue(os.path.exists(uploadedfile1_physical_file_path))

        request2 = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile2.txt', 'Test2'),
            'collectionid': collectionid,
            'mode': 'singlefile'
        })
        request2.user = self.testuser
        response2 = UploadTemporaryFilesView.as_view()(request2)
        self.assertEquals(response2.status_code, 200)
        response2data = json.loads(response2.content)
        self.assertEquals(collectionid, response2data['collectionid'])
        self.assertEquals(collection.files.count(), 1)
        uploadedfile2 = collection.files.first()
        self.assertEqual(uploadedfile2.filename, 'testfile2.txt')
        self.assertEqual(uploadedfile2.file.read(), 'Test2')
        self.assertFalse(os.path.exists(uploadedfile1_physical_file_path))

    def test_post_accept_ok(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'accept': 'text/plain,application/pdf'
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        collectionid = responsedata['collectionid']

        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.accept, 'text/plain,application/pdf')
        self.assertEquals(collection.files.count(), 1)
        uploadedfile = collection.files.first()
        self.assertEqual(uploadedfile.mimetype, 'text/plain')

        self.assertEquals(responsedata['temporaryfiles'][0]['mimetype'], 'text/plain')

    def test_post_accept_invalid_single_file(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'accept': 'application/pdf'
        })
        request.user = self.testuser
        self.assertEquals(TemporaryFileCollection.objects.count(), 0)
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 400)
        responsedata = json.loads(response.content)
        self.assertEquals(TemporaryFileCollection.objects.count(), 0)
        self.assertEquals(responsedata['file'][0]['code'], 'unsupported_mimetype')
        self.assertEquals(responsedata['file'][0]['message'], u'testfile1.txt: Unsupported filetype.')

    def test_post_accept_invalid_existing_collection(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(
            collection=collection,
            filename='testfile.txt')
        temporaryfile.save()

        request = self.factory.post('/test', {
            'collectionid': collection.id,
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'accept': 'application/pdf'
        })
        request.user = self.testuser
        self.assertEquals(TemporaryFileCollection.objects.count(), 1)
        self.assertEquals(collection.files.count(), 1)
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 400)
        responsedata = json.loads(response.content)
        self.assertEquals(TemporaryFileCollection.objects.count(), 1)
        self.assertEquals(collection.files.count(), 1)
        self.assertEquals(responsedata['file'][0]['code'], 'unsupported_mimetype')
        self.assertEquals(responsedata['file'][0]['message'], u'testfile1.txt: Unsupported filetype.')

    def test_post_accept_invalid_multiple_files(self):
        request = self.factory.post('/test', {
            'file': [
                SimpleUploadedFile('testfile1.pdf', 'Test1'),
                SimpleUploadedFile('testfile2.txt', 'Test2'),
            ],
            'accept': 'application/pdf'
        })
        request.user = self.testuser
        self.assertEquals(TemporaryFileCollection.objects.count(), 0)
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 400)
        responsedata = json.loads(response.content)
        self.assertEquals(TemporaryFileCollection.objects.count(), 0)
        self.assertEquals(responsedata['file'][0]['code'], 'unsupported_mimetype')
        self.assertEquals(responsedata['file'][0]['message'], u'testfile2.txt: Unsupported filetype.')

    def test_post_max_filename_length_not_truncated(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('abc.txt', 'Testcontent'),
            'max_filename_length': '7'
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        collectionid = responsedata['collectionid']

        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.files.first().filename, 'abc.txt')
        self.assertEquals(responsedata['temporaryfiles'][0]['filename'], 'abc.txt')

    def test_post_max_filename_length_truncated(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('abc.txt', 'Testcontent'),
            'max_filename_length': '6'
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        collectionid = responsedata['collectionid']

        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEquals(collection.files.first().filename, 'bc.txt')
        self.assertEquals(responsedata['temporaryfiles'][0]['filename'], 'bc.txt')

    def test_post_unique_filenames(self):
        request = self.factory.post('/test', {
            'file': [
                SimpleUploadedFile('abc.txt', 'Testcontent1'),
                SimpleUploadedFile('abc.txt', 'Testcontent2'),
            ],
            'unique_filenames': True
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        collectionid = responsedata['collectionid']
        collection = TemporaryFileCollection.objects.get(id=collectionid)

        files_by_content = {}
        for temporaryfile in collection.files.all():
            files_by_content[temporaryfile.file.read()] = temporaryfile
        self.assertEquals(files_by_content['Testcontent1'].filename, 'abc.txt')
        self.assertNotEquals(files_by_content['Testcontent2'].filename, 'abc.txt')
        self.assertTrue(files_by_content['Testcontent2'].filename.endswith('abc.txt'))

        self.assertEquals(responsedata['temporaryfiles'][0]['filename'], 'abc.txt')
        self.assertNotEquals(responsedata['temporaryfiles'][1]['filename'], 'abc.txt')
        self.assertTrue(responsedata['temporaryfiles'][1]['filename'].endswith('abc.txt'))

    def test_post_unique_filenames_and_max_filename_length(self):
        request = self.factory.post('/test', {
            'file': [
                SimpleUploadedFile('t'*50, 'Testcontent1'),
                SimpleUploadedFile('t'*50, 'Testcontent2'),
            ],
            'unique_filenames': True,
            'max_filename_length': 45
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)
        collectionid = responsedata['collectionid']
        collection = TemporaryFileCollection.objects.get(id=collectionid)

        files_by_content = {}
        for temporaryfile in collection.files.all():
            files_by_content[temporaryfile.file.read()] = temporaryfile
        self.assertEquals(files_by_content['Testcontent1'].filename,
                          'ttttttttttttttttttttt...ttttttttttttttttttttt')
        self.assertNotEquals(files_by_content['Testcontent2'].filename,
                             'ttttttttttttttttttttt...ttttttttttttttttttttt')
        self.assertTrue(files_by_content['Testcontent2'].filename.endswith('-tttttttt'))

    def test_post_form_invalid_no_file(self):
        request = self.factory.post('/test')
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 400)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['file'][0]['code'], 'required')

    def test_post_form_invalid_collectionid_not_number(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'collectionid': 'invalid'
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 400)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['collectionid'][0]['code'], 'invalid')

    def test_post_form_collectionid_does_not_exist(self):
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'collectionid': '10001'
        })
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 404)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['collectionid'][0]['code'], 'doesnotexist')

    def test_post_form_collection_not_owned_by_user(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        request = self.factory.post('/test', {
            'file': SimpleUploadedFile('testfile1.txt', 'Test1'),
            'collectionid': collection.id
        })
        request.user = create_user('otheruser')
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 404)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['collectionid'][0]['code'], 'doesnotexist')

    def test_delete_single_file(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(
            collection=collection,
            filename='testfile.txt')
        temporaryfile.file.save('testfile.txt', ContentFile('testcontent'))

        request = self.factory.delete('/test', json.dumps({
            'collectionid': collection.id,
            'temporaryfileid': temporaryfile.id
        }))
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 200)
        responsedata = json.loads(response.content)

        self.assertEquals(responsedata['collectionid'], collection.id)
        self.assertEquals(responsedata['temporaryfileid'], temporaryfile.id)

        self.assertTrue(TemporaryFileCollection.objects.filter(id=collection.id).exists())
        self.assertFalse(TemporaryFile.objects.filter(id=temporaryfile.id).exists())

    def test_delete_invalid_json_reqeust(self):
        request = self.factory.delete('/test', data='invalid')
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 400)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['errormessage'], 'Invalid JSON data in the request body.')

    def test_delete_form_collectionid_does_not_exist(self):
        request = self.factory.delete('/test', data=json.dumps({
            'collectionid': '10001',
            'temporaryfileid': '1'  # NOTE: This is ignored since we never get to the code looking it up
        }))
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 404)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['collectionid'][0]['code'], 'doesnotexist')

    def test_delete_form_collection_not_owned_by_user(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        request = self.factory.delete('/test', json.dumps({
            'collectionid': collection.id,
            'temporaryfileid': '1'  # NOTE: This is ignored since we never get to the code looking it up
        }))
        request.user = create_user('otheruser')
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 404)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['collectionid'][0]['code'], 'doesnotexist')

    def test_delete_form_temporaryfileid_does_not_exist(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        request = self.factory.delete('/test', data=json.dumps({
            'collectionid': collection.id,
            'temporaryfileid': 10001
        }))
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEquals(response.status_code, 404)
        responsedata = json.loads(response.content)
        self.assertEquals(responsedata['temporaryfileid'][0]['code'], 'doesnotexist')



from django.conf.urls import url
from django.conf.urls import patterns
from functools import update_wrapper

from .decorators import cradminview


#: The name of the app index view (the landing page for the app).
#: We do not enforce this, but we assume that each app has a
#: view with this name.
INDEXVIEW_NAME = 'INDEX'


class Url(object):
    """
    Url is mostly the same as func:`django.conf.urls.url`.
    You use Url to add urls to an app.
    """
    def __init__(self, regex, view, kwargs=None, name=None):
        self.regex = regex
        self.view = view
        self.kwargs = kwargs
        self.name = name


class App(object):
    appurls = []

    def __init__(self, appname, request):
        self.appname = appname
        self.request = request

    def reverse_appurl(self, viewname, args=None, kwargs=None):
        """
        Works just like :func:`django.core.urlresolvers.reverse`, except
        that the name is the name given in :obj:`.appurls`, not the full
        name of the URL.

        This means that you should use this to reverse urls within this app.
        """
        return self.request.cradmin_instance.reverse_url(
            appname=self.appname, viewname=viewname,
            args=args, kwargs=kwargs)

    def reverse_appindexurl(self):
        """
        Shortcut for::

            from django_cradmin import crapp
            cradmin_app.reverse_appurl(crapp.INDEXVIEW_NAME)
        """
        return self.reverse_appurl(viewname=INDEXVIEW_NAME)

    @classmethod
    def _wrap_view(cls, appname, view):
        def viewwrapper(request, *args, **kwargs):
            request.cradmin_app = cls(appname, request)
            return cradminview(view)(request, *args, **kwargs)

        # Take name and docstring from view
        update_wrapper(viewwrapper, view, updated=())

        return viewwrapper

    @classmethod
    def build_urls(cls, cradmin_instance_id, appname):
        """
        Used internally by :meth:`django_cradmin.crinstance.BaseCrAdminInstance.urls`
        to build urls for all views in the app.
        """
        urls = []
        for pattern in cls.appurls:
            urls.append(
                url(
                    pattern.regex, cls._wrap_view(appname, pattern.view),
                    name='{}-{}-{}'.format(cradmin_instance_id, appname, pattern.name),
                    kwargs=pattern.kwargs))
        return patterns('', *urls)



from django.conf.urls import patterns, url


urlpatterns = patterns(
    'django_cradmin.views.auth',
    url(r'^login$', 'login.cradmin_loginview', name='django_cradmin-login'),
    url(r'^logout$', 'logout.cradmin_logoutview', name='django_cradmin-logout'),

    # url(r'^signup/$', SignupView.as_view(), name='django_cradmin-signup'),
    # url(r'^signup/success$', SignupSuccessView.as_view(), name='django_cradmin-signup-success'),
    # url(r'^signup/activate/(?P<key>[a-zA-Z0-9:_%+.-]+)$', SignupActivateView.as_view(),
    #     name='django_cradmin-signup-activate'),
    #
    # url(r'^password_reset/$',
    #     'views.password_reset.password_reset_view',
    #     name='django_cradmin-password_reset'),
    # url(r'^password_reset/done$',
    #     'views.password_reset.password_reset_done_view',
    #     name='django_cradmin-password_reset_done'),
    # url(r'^password_reset/confirm/(?P<uidb64>[0-9A-Za-z=+_]+)-(?P<token>.+)$',
    #     'views.password_reset.password_reset_confirm_view',
    #     name='django_cradmin-password_reset_confirm'),
    # url(r'^password_reset/complete$',
    #     'views.password_reset.password_reset_complete_view',
    #     name='django_cradmin-password_reset_complete')
)



from django import test

from django_cradmin.tests.sortable.cradmin_sortable_testapp.models import SortableItem
from django_cradmin.tests.sortable.cradmin_sortable_testapp.models import ItemContainer


class TestSortableItem(test.TestCase):
    """
    Test that classes inherit :class:`SortableBase` works as advertised!
    """

    def _create_container(self):
        container = ItemContainer(name='test container')
        container.save()
        return container

    def _create_items(self, num_items, container=None):
        container = container or self._create_container()
        items = []
        for i in range(1, num_items+1, 1):
            items.append(SortableItem(
                container=container,
                name='test{}'.format(i),
                sort_index=i))
        SortableItem.objects.bulk_create(items)
        return SortableItem.objects.all()

    def test_has_sort_index_field(self):
        item = self._create_items(1)[0]
        self.assertTrue(hasattr(item, 'sort_index'))
        self.assertTrue(item.sort_index, 1)

    def test_sort_item_last(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_last(i1)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [2, 3, 4, 5, 6, 1])

    def test_sort_item_3_last(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_last(i3)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [1, 2, 4, 5, 6, 3])

    def test_sort_last_item_first(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i6, i1.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [6, 1, 2, 3, 4, 5])

    def test_sort_item_3_first(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i3, i1.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [3, 1, 2, 4, 5, 6])

    def test_sort_item_move_first_before_id_4(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i1, i4.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [2, 3, 1, 4, 5, 6])

    def test_sort_item_move_second_before_id_4(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i2, i4.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [1, 3, 2, 4, 5, 6])

    def test_sort_item_move_second_before_id_4_in_allready_reordered_list(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_last(i1)
        # now list is [2,3,4,5,6,1]
        SortableItem.objects.sort_before(i2, i4.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [3, 2, 4, 5, 6, 1])

    def test_set_sort_index_to_last_no_siblings(self):
        container = self._create_container()
        item = SortableItem(container=container, name='My test')
        SortableItem.objects.set_newitem_sort_index_to_last(item)
        self.assertEquals(item.sort_index, 1)
        self.assertIsNone(item.pk)  # The item was not saved

    def test_set_sort_index_to_last_has_siblings(self):
        container = self._create_container()
        self._create_items(2, container=container)
        item = SortableItem(container=container, name='My test')
        SortableItem.objects.set_newitem_sort_index_to_last(item)
        self.assertEquals(item.sort_index, 3)
        self.assertIsNone(item.pk)  # The item was not saved

    def test_set_sort_index_to_last_refuse_with_sort_index(self):
        container = self._create_container()
        item = SortableItem(container=container, name='My test', sort_index=1)
        with self.assertRaises(ValueError):
            SortableItem.objects.set_newitem_sort_index_to_last(item)

    def test_set_sort_index_to_last_refuse_existing(self):
        container = self._create_container()
        item = SortableItem(container=container, name='My test', sort_index=1)
        item.save()
        item.sort_index = None
        with self.assertRaises(ValueError):
            SortableItem.objects.set_newitem_sort_index_to_last(item)



from django.http import HttpResponseRedirect
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.views import login
from django import forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit


class CradminAuthenticationForm(AuthenticationForm):
    username = forms.CharField(max_length=254, label=_('Email'))

    error_messages = {
        'invalid_login': _("Your email and password didn't match. Please try again."),
        'inactive': _("This account is inactive."),
    }

    def __init__(self, *args, **kwargs):
        super(CradminAuthenticationForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.add_input(Submit('login', _('Sign in')))


def cradmin_loginview(request, template_name='django_cradmin/auth/login.django.html',
                      authentication_form=CradminAuthenticationForm):
    if request.user.is_authenticated():
        return HttpResponseRedirect(settings.LOGIN_REDIRECT_URL)

    return login(
        request,
        template_name=template_name,
        authentication_form=authentication_form)



from django.utils.translation import ugettext_lazy as _
from django.views.generic import UpdateView as DjangoUpdateView

from django_cradmin.crispylayouts import PrimarySubmit
from django_cradmin.crispylayouts import DefaultSubmit
from .crudbase import CreateUpdateViewMixin


class UpdateView(CreateUpdateViewMixin, DjangoUpdateView):
    template_name = 'django_cradmin/viewhelpers/update.django.html'

    def get_buttons(self):
        buttons = [
            PrimarySubmit('submit-save', _('Save')),
            DefaultSubmit('submit-save-and-continue-editing', _('Save and continue editing')),
        ]
        preview_url = self.get_preview_url()
        if preview_url:
            buttons.append(DefaultSubmit('submit-preview', _('Preview')))
        return buttons

    def get_formhelper(self):
        helper = super(UpdateView, self).get_formhelper()
        helper.form_id = 'django_cradmin_updateform'
        return helper

    def get_queryset_for_role(self, role):
        """
        Get a queryset with all objects of :obj:`.model`  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role(self.request.cradmin_role)
        return queryset

    def get_success_message(self, object):
        return _('Saved "%(object)s".') % {'object': object}
